#!/bin/sh -e

if [ -x /bin/realpath ] || [ -x /usr/bin/realpath ]; then
  PWD0=`dirname $0`
  PWD=`realpath "$PWD0"`
else
  PWD0=`dirname $0`
  OLDPWD=`pwd`
  cd "$PWD0"
  PWD=`pwd`
  cd "$OLDPWD"
fi

if [ ! -x "scripts/BUILD.CONF" ]; then
  echo "first-run initializing"
  if ( uname -a | grep Linux >> /dev/null ) ; then
    echo "OS: Linux"
    cp scripts/BUILD.CONF.Linux scripts/BUILD.CONF.tmp
  elif ( uname -a | grep GNU >> /dev/null ) ; then
    echo "OS: non-Linux GNU"
    cp scripts/BUILD.CONF.Linux scripts/BUILD.CONF.tmp
  elif ( uname -a | grep FreeBSD >> /dev/null ) ; then
    echo "OS: FreeBSD"
    cp scripts/BUILD.CONF.FreeBSD scripts/BUILD.CONF.tmp
  elif ( uname -a | grep OpenBSD >> /dev/null ) ; then
    echo "OS: OpenBSD"
    cp scripts/BUILD.CONF.OpenBSD scripts/BUILD.CONF.tmp
  fi
  
  if [ -f /usr/bin/gcc ]; then
    echo "compiler: /usr/bin/gcc"
  elif [ -f /usr/local/bin/gcc ]; then
    echo "compiler: /usr/local/bin/gcc"
    echo "CC=\"/usr/local/bin/gcc\"" >> scripts/BUILD.CONF.tmp
    echo "LD=\"/usr/local/bin/gcc\"" >> scripts/BUILD.CONF.tmp
  elif [ -f /usr/bin/cc ]; then
    echo "compiler: /usr/bin/cc (clang?)"
    echo "CC=\"/usr/bin/cc\"" >> scripts/BUILD.CONF.tmp
    echo "LD=\"/usr/bin/cc\"" >> scripts/BUILD.CONF.tmp
     # disable stupid clang-warnings
    echo "WARNOPTS=\"\$WARNOPTS -Wno-empty-body\"" >> scripts/BUILD.CONF.tmp
  fi
  
  echo "BPREFIX=\"$PWD/..\"" >> scripts/BUILD.CONF.tmp
  echo "BINPATH=\"$PWD/../bin\"" >> scripts/BUILD.CONF.tmp
  mv scripts/BUILD.CONF.tmp scripts/BUILD.CONF
fi

DIR="$1"
shift

if [ -z "$DIR" ]; then
  echo "Usage: ./build.sh project_dir"
  exit 1
fi

if [ ! -d "$DIR" ]; then
  echo "directory $DIR not found"
  exit 1
fi

export PATH="$PWD/scripts:$PATH"

cd "$DIR"
./build.sh "$@"
