#include <stdio.h>
#include <string.h>


typedef int (*fn_opthandle)(char const * v);

typedef struct {
  char const * name;
  fn_opthandle handler;
  int allow_dup;
  char const * desc;
  int present;
} fcl_option;

static int opt_strerror(char const * v);
static fcl_option opts[] = {
  { "strerror", &opt_strerror, 0,
    "={no|static|dynamic}\n"
    "strerror() caching mode:\n"
    "  no - no cache, use strerror() from libc\n"
    "  static - use only compile-time pregenerated cache\n"
    "  dynamic - allow run-time cache regeneration via fcl_strerror_reinit()\n"
    },
  { NULL }
};

static void print_help(void) {
  fcl_option * o;
  char const * d;
  unsigned int j, f;
  fprintf(stderr, "Available configuration options:\n");
  for(o=opts; o->name; o++) {
    fprintf(stderr, "  --%s", o->name);
    if(!(d = o->desc)) { fputc('\n', stderr); continue; }
    if(*d=='=') f = 1; else { f = 0; fputc('\n', stderr); }
    while(*d) {
      for(j=0; d[j] && d[j]!='\n'; j++);
      if(f) f=0; else fputs("        ",stderr);
      fwrite(d, 1, j, stderr);
      fputc('\n',stderr);
      d+=j;
      if(*d) d++;
    }
  }
}

static char const * sp(char const * a, char const * p) {
  char c;
  while(c=*(p++)) {
    if(c!=*a) return NULL;
    a++;
  }
  return a;
}

static int opt_strerror(char const * v) {
  if(!strcmp(v,"no")) puts("#define FCL_NO_STRERROR");
  else if(!strcmp(v,"static")) puts("#define FCL_STRERROR_STATIC");
  else if(!strcmp(v,"dynamic")) ;
  else return -2;
  return 0;
}


int main(int argc, char * * argv) {
  int j, w, r;
  char *a, *v;
  fcl_option * o;
  
  for(w=0,j=1; j<argc; j++) {
    a = argv[j];
    if(!strcmp(a,"--help")) {
      print_help();
      return -1;
    }
    if(a[0]!='-' || a[1]!='-') {
      fprintf(stderr, "[opts-config] bad argument: %s\n", a);
      return -1;
    }
    a += 2;
    if(v = strchr(a,'=')) *(v++) = 0;
    for(o=opts; o->name; o++) if(!strcmp(a, o->name)) break;
    if(!o->name) {
      fprintf(stderr, "[opts-config] unknown option: --%s\n", a);
      return -1;
    }
    if(!w) { w=1; puts("/* command-line specified options - begin */"); }
    if(!o->allow_dup && o->present) {
      fprintf(stderr, "[opts-config] duplicate option: --%s\n", a);
      return -1;
    }
    if(o->desc && *o->desc=='=' && !v) {
      fprintf(stderr, "[opts-config] option needs parameter: --%s\n", a);
      return -1;
    }
    if((!o->desc || *o->desc!='=') && v) {
      fprintf(stderr, "[opts-config] option doesn't need parameter: --%s\n", a);
      return -1;
    }
    o->present = 1;
    r = o->handler(v);
    if(r<0) {
      fprintf(stderr, "Error processing option: --%s\n", a);
      return -1;
    }
  }
  if(w) puts("/* command-line specified options - end */\n");
  return 0;
}

