/*
 * This file is part of firk's Common C Library
 *  gentypes.c - generate types.h
 *
 * (c) firk 2015, 2020-2022; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#include <stdio.h>

static char const *n8, *n16, *n32, *n64;
int main(void) {
  unsigned char two[2];
  unsigned int twox;
#define CHECK1(n) if(sizeof(n)!=sizeof(signed n) || sizeof(n)!=sizeof(unsigned n)) { fputs("> error: sizeof(" #n ") inconsistent\n", stderr); return -1; }
  CHECK1(char)
  CHECK1(short)
  CHECK1(int)
  CHECK1(long)
#ifdef HAVE_LONGLONG
  CHECK1(long long)
#endif

  if(sizeof(signed char)!=1 || sizeof(unsigned char)!=1 || sizeof(char)!=1) { fputs("> error: sizeof(char)!=1\n", stderr); return -1; }
  n8 = "char";

  if(sizeof(signed short)!=2 || sizeof(unsigned short)!=2 || sizeof(short)!=2) { fputs("> error: sizeof(short)!=2\n", stderr); return -1; }
  n16 = "short";

  if(sizeof(int)==4) {
    if(sizeof(signed int)!=4 || sizeof(unsigned int)!=4) { fputs("> error: sizeof(int) inconsistent\n", stderr); return -1; }
    n32 = "int";
  } else if(sizeof(long)==4) {
    if(sizeof(signed long)!=4 || sizeof(unsigned long)!=4) { fputs("> error: sizeof(long) inconsistent\n", stderr); return -1; }
    n32 = "long";
#ifndef ALLOW_MISSING32
  } else {
    fputs("> error: missing int32 type\n", stderr); return -1;
#endif
  }

#ifdef HAVE_LONGLONG
  if(sizeof(long long)==8) {
    if(sizeof(signed long long)!=8 || sizeof(unsigned long long)!=8) { fputs("> error: sizeof(long long) inconsistent\n", stderr); return -1; }
    n64 = "long long";
  } else
#endif
  if(sizeof(long)==8) {
    if(sizeof(signed long)!=8 || sizeof(unsigned long)!=8) { fputs("> error: sizeof(long) inconsistent\n", stderr); return -1; }
    n64 = "long";
  }

#ifdef HAVE_LONGLONG
  fputs("#define HAVE_LONGLONG\n", stdout);
#endif
  if(n8) fputs("#define HAVE_INT8\n", stdout);
  if(n16) fputs("#define HAVE_INT16\n", stdout);
  if(n32) fputs("#define HAVE_INT32\n", stdout);
  if(n64) fputs("#define HAVE_INT64\n", stdout);
  if(sizeof(int)>=4) fputs("#define INT_GE32\n", stdout);
  if(sizeof(long)>=8) fputs("#define LONG_GE64\n", stdout);
  fputs("/* int8=schar always; uint8=uchar always; other may vary */\n", stdout);
  if(n8) printf("typedef signed %s int8, schar;\n", n8);
  if(n16) printf("typedef signed %s int16;\n", n16);
  if(n32) printf("typedef signed %s int32;\n", n32);
  if(n64) printf("typedef signed %s int64;\n", n64);
  if(n8) printf("typedef unsigned %s uint8, uchar;\n", n8);
  if(n16) printf("typedef unsigned %s uint16;\n", n16);
  if(n32) printf("typedef unsigned %s uint32;\n", n32);
  if(n64) printf("typedef unsigned %s uint64;\n", n64);
  fputs("typedef unsigned short ushort;\n"
        "typedef unsigned int uint;\n"
        "typedef unsigned long ulong;\n"
#ifdef HAVE_LONGLONG
        "typedef signed long long llong;\n"
        "typedef unsigned long long ullong;\n"
#endif
        , stdout);
  two[0] = 10; two[1] = 20; twox = *(unsigned short*)two;
  if(twox == 0x140A) printf("#define FCL_LE 1\n");
  else if(twox == 0x0A14) printf("#define FCL_BE 1\n");
  else { fprintf(stderr, "> error: weird endianness: 0x0A, 0x14 = 0x%04X\n", twox); return -1; }
#ifdef HAVE_LONGLONG
  fputs("> have 'long long'? yes\n", stderr);
#else
  fputs("> have 'long long'? no\n", stderr);
#endif
  fputs("> types.h created ok\n", stderr);
  return 0;
}

