#include <stdio.h>
#include "types.h"

#define ASSERTT(X,T) do { if(X) { if(verbose) printf("> OK: %s\n", T ); } else { printf("> FAIL: %s\n", T); fail=1; if(!all) return -1; } } while(0)
#define ASSERT(X) do { if(X) { if(verbose) printf("> OK: %s\n", #X ); } else { printf("> FAIL: %s\n", #X ); fail=1; if(!all) return -1; } } while(0)
int main(void) {
  int fail = 0;
  union { char c; uchar u; schar s; } uuu;

  printf("testing system configuration...\n");
  ASSERT(sizeof(char)==1);
  ASSERT(sizeof(schar)==1);
  ASSERT(sizeof(uchar)==1);
  ASSERT(sizeof(short)>=2);
  ASSERT(sizeof(signed short)==sizeof(short));
  ASSERT(sizeof(unsigned short)==sizeof(short));
  ASSERT(sizeof(int)>=2);
  ASSERT(sizeof(int)>=sizeof(short));
  ASSERT(sizeof(signed int)==sizeof(int));
  ASSERT(sizeof(unsigned int)==sizeof(int));
  ASSERT(sizeof(long)>=4);
  ASSERT(sizeof(long)>=sizeof(int));
  ASSERT(sizeof(signed long)==sizeof(long));
  ASSERT(sizeof(unsigned long)==sizeof(long));
#ifdef HAVE_LONGLONG
  ASSERT(sizeof(long long)>=8);
  ASSERT(sizeof(long long)>=sizeof(long));
  ASSERT(sizeof(signed long long)==sizeof(long long));
  ASSERT(sizeof(unsigned long long)==sizeof(long long));
#endif
#ifdef HAVE_INT8
  ASSERT(sizeof(int8)==1);
  ASSERT(sizeof(uint8)==1);
#endif
#ifdef HAVE_INT16
  ASSERT(sizeof(int16)==2);
  ASSERT(sizeof(uint16)==2);
#endif
#ifdef HAVE_INT32
  ASSERT(sizeof(int32)==4);
  ASSERT(sizeof(uint32)==4);
#endif
#ifdef HAVE_INT64
  ASSERT(sizeof(int64)==8);
  ASSERT(sizeof(uint64)==8);
#endif
  if(((char)-1)<0) {
    printf("> GOT: raw char is signed\n");
    ASSERT(((long)(char)-128)==-128);
    ASSERT(((long)(char)-129)==127);
    ASSERT(((long)(char)128)==-128);
  } else {
    printf("> GOT: raw char is unsigned\n");
    ASSERT(((long)(char)-1)==255);
    ASSERT(((long)(char)-128)==128);
    ASSERT(((long)(char)-129)==127);
  }
  ASSERT(((long)(schar)-128)==-128);
  ASSERT(((long)(schar)-129)==127);
  ASSERT(((long)(schar)128)==-128);
  ASSERT(((long)(uchar)-1)==255);
  ASSERT(((long)(uchar)-128)==128);
  ASSERT(((long)(uchar)-129)==127);
#ifdef HAVE_INT16
  ASSERT(((long)(int16)-1)==-1);
  ASSERT(((long)(int16)65535)==-1);
  ASSERT(((long)(int16)-32768)==-32768);
  ASSERT(((long)(int16)32768)==-32768);
  ASSERT(((long)(uint16)-1)==65535);
  ASSERT(((long)(uint16)65535)==65535);
  ASSERT(((long)(uint16)-32768)==32768);
  ASSERT(((long)(uint16)32768)==32768);
#endif
  uuu.c = 0;
  ASSERTT(uuu.u==0 && uuu.s==0, "bits: (char)0 => uchar 0, schar 0");
  uuu.c = 256;
  ASSERTT(uuu.u==0 && uuu.s==0, "bits: (char)256 => uchar 0, schar 0");
  ASSERTT(uuu.u!=256 && uuu.s!=256, "bits: (char)256 => NOT uchar 256, NOT schar 256");
  uuu.c = 1;
  ASSERTT(uuu.u==1 && uuu.s==1, "bits: (char)1 => uchar 1, schar 1");
  uuu.c = 127;
  ASSERTT(uuu.u==127 && uuu.s==127, "bits: (char)127 => uchar 127, schar 127");
  uuu.c = 128;
  ASSERTT(uuu.u==128 && uuu.s==-128, "bits: (char)128 => uchar 128, schar -128");
  ASSERTT(uuu.u!=-128 && uuu.s!=128, "bits: (char)128 => NOT uchar -128, NOT schar 128");
  uuu.c = -128;
  ASSERTT(uuu.u==128 && uuu.s==-128, "bits: (char)-128 => uchar 128, schar -128");
  ASSERTT(uuu.u!=-128 && uuu.s!=128, "bits: (char)-128 => NOT uchar -128, NOT schar 128");
  uuu.c = 255;
  ASSERTT(uuu.u==255 && uuu.s==-1, "bits: (char)255 => uchar 255, schar -1");
  uuu.c = -1;
  ASSERTT(uuu.u==255 && uuu.s==-1, "bits: (char)-1 => uchar 255, schar -1");
  if(fail) { printf("> SOMETHING BAD!\n"); return -1; }
  printf("> ALL CHECKS PASSED!\n");
  return 0;
}
