/*
 * This file is part of firk's Common C Library
 *  conio/lowgetch.c - getch() directly by reading stdin
 *
 * (c) firk 2015; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#include <termios.h>
#include <unistd.h>
#include <errno.h>
#include <sys/select.h>
#include <sys/ioctl.h>
#include "conio.h"

#ifndef STDIN_FILENO
#define STDIN_FILENO 0
#endif
#ifndef STDOUT_FILENO
#define STDOUT_FILENO 1
#endif

int keypressed_low(void) {
  struct timeval tv;
  fd_set rfds;
  tv.tv_sec = 0;
  tv.tv_usec = 0;
  FD_ZERO(&rfds);
  FD_SET(STDIN_FILENO, &rfds);

  if(select(STDIN_FILENO+1, &rfds, NULL, NULL, &tv)!=1) return 0;
  return FD_ISSET(STDIN_FILENO, &rfds);
}

int keypressed_wait_low(int usec) {
  struct timeval tv;
  fd_set rfds;
  tv.tv_sec = 0;
  tv.tv_usec = usec;
  FD_ZERO(&rfds);
  FD_SET(STDIN_FILENO, &rfds);
  if(select(STDIN_FILENO+1, &rfds, NULL, NULL, &tv)!=1) return 0;
  return FD_ISSET(STDIN_FILENO, &rfds);
  /* TODO: may return fake negative due to EINTR */
}

int getch_raw_low(void) {
  unsigned char c;
  int res;
  while(1) {
    res = read(STDIN_FILENO, &c, 1);
    if(res==1) return c;
    if(res<0 && errno==EINTR) continue;
    break;
  }
  return EOF;
  /* wait/non-wait depending on O_NONBLOCK flag on stdin */
  /* return EOF for non-blocking input if no data */
}


