/*
 * This file is part of firk's Common C Library
 *  endian.c - endian-dependent byte-array utils
 *
 * (c) firk 2021; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#include "types.h"
#include "endian.h"


#define SW16(val) \
    ((val & 0xFF00) >> 8) | \
    ((val & 0x00FF) << 8)

#define SW32(val) \
    ((val & 0xFF000000) >> 24) | \
    ((val & 0x00FF0000) >> 8) | \
    ((val & 0x0000FF00) << 8) | \
    ((val & 0x000000FF) << 24)

#define SW64(val) \
    ((val & 0xFF00000000000000) >> 56) | \
    ((val & 0x00FF000000000000) >> 40) | \
    ((val & 0x0000FF0000000000) >> 24) | \
    ((val & 0x000000FF00000000) >> 8) | \
    ((val & 0x00000000FF000000) << 8) | \
    ((val & 0x0000000000FF0000) << 24) | \
    ((val & 0x000000000000FF00) << 40) | \
    ((val & 0x00000000000000FF) << 56)

extern uint16 BSWAP16(uint16 val)
  { return SW16(val); }
extern uint16 U16SWAP(uchar const * buf)
  { uint16 val; val = *(uint16*)buf; return SW16(val); }
extern void setU16SWAP(uchar * buf, uint16 val)
  { *(uint16*)buf = SW16(val); }

extern uint32 BSWAP32(uint32 val)
  { return SW32(val); }
extern uint32 U32SWAP(uchar const * buf)
  { uint32 val; val = *(uint32*)buf; return SW32(val); }
extern void setU32SWAP(uchar * buf, uint32 val)
  { *(uint32*)buf = SW32(val); }

#ifdef HAVE_INT64
extern uint64 BSWAP64(uint64 val)
  { return SW64(val); }
extern uint64 U64SWAP(uchar const * buf)
  { uint64 val; val = *(uint64*)buf; return SW64(val); }
extern void setU64SWAP(uchar * buf, uint64 val)
  { *(uint64*)buf = SW64(val); }
#endif


