/*
 * This file is part of firk's Common C Library
 *  endian.h - endian-dependent byte-array utils
 *
 * (c) firk 2021; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef FCL_INCLUDED_ENDIAN_H
#define FCL_INCLUDED_ENDIAN_H

#define U8(buf) (*(uint8*)(buf))
#define getU8(buf, val) do { (val) = U8(buf); } while(0)
#define setU8(buf, val) do { (*(uint8*)(buf)) = (val); } while(0)
#ifdef FCL_LE
/* LE on LE */
#define U16LE(buf) (*(uint16*)(buf))
#define getU16LE(buf, val) do { (val) = U16LE(buf); } while(0)
#define setU16LE(buf, val) do { (*(uint16*)(buf)) = (val); } while(0)
#define U32LE(buf) (*(uint32*)(buf))
#define getU32LE(buf, val) do { (val) = U32LE(buf); } while(0)
#define setU32LE(buf, val) do { (*(uint32*)(buf)) = (val); } while(0)
#ifdef HAVE_INT64
#define U64LE(buf) (*(uint64*)(buf))
#define getU64LE(buf, val) do { (val) = U64LE(buf); } while(0)
#define setU64LE(buf, val) do { (*(uint64*)(buf)) = (val); } while(0)
#endif
/* BE on LE */
#define U16BE(buf) U16SWAP((uchar*)(buf))
#define getU16BE(buf, val) do { (val) = U16BE(buf); } while(0)
#define setU16BE(buf, val) setU16SWAP((uchar*)(buf), val)
#define U32BE(buf) U32SWAP((uchar*)(buf))
#define getU32BE(buf, val) do { (val) = U32BE(buf); } while(0)
#define setU32BE(buf, val) setU32SWAP((uchar*)(buf), val)
#ifdef HAVE_INT64
#define U64BE(buf) U64SWAP((uchar*)(buf))
#define getU64BE(buf, val) do { (val) = U64BE(buf); } while(0)
#define setU64BE(buf, val) setU64SWAP((uchar*)(buf), val)
#endif
/* ... */
#else
#ifdef FCL_BE
/* BE on BE */
#define U16BE(buf) (*(uint16*)(buf))
#define getU16BE(buf, val) do { (val) = U16BE(buf); } while(0)
#define setU16BE(buf, val) do { (*(uint16*)(buf)) = (val); } while(0)
#define U32BE(buf) (*(uint32*)(buf))
#define getU32BE(buf, val) do { (val) = U32BE(buf); } while(0)
#define setU32BE(buf, val) do { (*(uint32*)(buf)) = (val); } while(0)
#ifdef HAVE_INT64
#define U64BE(buf) (*(uint64*)(buf))
#define getU64BE(buf, val) do { (val) = U64BE(buf); } while(0)
#define setU64BE(buf, val) do { (*(uint64*)(buf)) = (val); } while(0)
#endif
/* LE on BE */
#define U16LE(buf) U16SWAP((uchar*)(buf))
#define getU16LE(buf, val) do { (val) = U16LE(buf); } while(0)
#define setU16LE(buf, val) setU16SWAP((uchar*)(buf), val)
#define U32LE(buf) U32SWAP((uchar*)(buf))
#define getU32LE(buf, val) do { (val) = U32LE(buf); } while(0)
#define setU32LE(buf, val) setU32SWAP((uchar*)(buf), val)
#ifdef HAVE_INT64
#define U64LE(buf) U64SWAP((uchar*)(buf))
#define getU64LE(buf, val) do { (val) = U64LE(buf); } while(0)
#define setU64LE(buf, val) setU64SWAP((uchar*)(buf), val)
#endif
/* ... */
#else
#error unknown endianness
#endif
#endif

extern uint16 BSWAP16(uint16 val);
extern uint16 U16SWAP(uchar const * buf);
extern void setU16SWAP(uchar * buf, uint16 val);
extern uint32 BSWAP32(uint32 val);
extern uint32 U32SWAP(uchar const * buf);
extern void setU32SWAP(uchar * buf, uint32 val);
#ifdef HAVE_INT64
extern uint64 BSWAP64(uint64 val);
extern uint64 U64SWAP(uchar const * buf);
extern void setU64SWAP(uchar * buf, uint64 val);
#endif

#endif

