/*
 * This file is part of firk's Common C Library
 *  hashtbl.h - commonly used hashtables
 *
 * (c) firk 2021; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef FCL_INCLUDED_HASHTBL_H
#define FCL_INCLUDED_HASHTBL_H

typedef union {
  schar sc;
  uchar uc;
  short ss;
  ushort us;
  int si;
  uint ui;
  long sl;
  ulong ul;
#ifdef HAVE_LONGLONG
  llong sll;
  ullong ull;
#endif
#ifdef HAVE_INT8
  int8 s8;
  uint8 u8;
#endif
#ifdef HAVE_INT16
  int16 s16;
  uint16 u16;
#endif
#ifdef HAVE_INT32
  int32 s32;
  uint32 u32;
#endif
#ifdef HAVE_INT64
  int64 s64;
  uint64 u64;
#endif
  void * ptr;
} fcl_ht_value;

typedef struct fcl_ht_item {
  ulong h;
  char const * name;
  struct fcl_ht_item * next;
  fcl_ht_value v;
} fcl_ht_item;

typedef struct {
  fcl_size w;
  fcl_size n;
  fcl_ht_item * * d;
} fcl_ht_static;

typedef struct {
  fcl_size w;
  fcl_size n;
  fcl_ht_item * * d;
} fcl_ht_malloc;

typedef void (*fcl_free_cb)(void * p);
typedef struct {
  fcl_size w;
  fcl_size n;
  fcl_free_cb cb;
  fcl_ht_item * * d;
} fcl_ht_custom;

#define FN(n) fcl_ht_static_ ## n
extern int FN(init)(fcl_ht_static * ht, fcl_size w);
extern fcl_ht_value * FN(get)(fcl_ht_static * ht, char const * name);
extern fcl_ht_value * FN(getl)(fcl_ht_static * ht, char const *name, size_t l);
extern int FN(unset)(fcl_ht_static * ht, char const * name);
extern int FN(set)(fcl_ht_static * ht,
  char const * name, fcl_ht_value v, int is_old);
extern void FN(clear)(fcl_ht_static * ht);
extern void FN(free)(fcl_ht_static * ht);
#undef FN

#define FN(n) fcl_ht_malloc_ ## n
extern int FN(init)(fcl_ht_malloc * ht, fcl_size w);
extern fcl_ht_value * FN(get)(fcl_ht_malloc * ht, char const * name);
extern fcl_ht_value * FN(getl)(fcl_ht_malloc * ht, char const *name, size_t l);
extern int FN(unset)(fcl_ht_malloc * ht, char const * name);
extern int FN(set)(fcl_ht_malloc * ht,
  char const * name, fcl_ht_value v, int is_old);
extern void FN(clear)(fcl_ht_malloc * ht);
extern void FN(free)(fcl_ht_malloc * ht);
#undef FN

#define FN(n) fcl_ht_custom_ ## n
extern int FN(init)(fcl_ht_custom * ht, fcl_size w, fcl_free_cb cb);
extern fcl_ht_value * FN(get)(fcl_ht_custom * ht, char const * name);
extern fcl_ht_value * FN(getl)(fcl_ht_custom * ht, char const *name, size_t l);
extern int FN(unset)(fcl_ht_custom * ht, char const * name);
extern int FN(set)(fcl_ht_custom * ht,
  char const * name, fcl_ht_value v, int is_old);
extern void FN(clear)(fcl_ht_custom * ht);
extern void FN(free)(fcl_ht_custom * ht);
#undef FN

#endif

