/*
 * This file is part of firk's Common C Library
 *  malloc.h - overflow-safe versions of memory allocation funcs
 *
 * (c) firk 2019, 2020; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef FCL_INCLUDED_MALLOC_H
#define FCL_INCLUDED_MALLOC_H

extern void fcl_malloc_debug_dump(void);

#define FNLN char const * fn, unsigned int ln

#define fcl_malloc(A) fcl_malloc_debug(A,__FILE__,__LINE__)
extern void * fcl_malloc_debug(fcl_size sz, FNLN);
#define fcl_mallocz(A) fcl_mallocz_debug(A,__FILE__,__LINE__)
extern void * fcl_mallocz_debug(fcl_size sz, FNLN);
#define fcl_realloc(A,B) fcl_realloc_debug(A,B,__FILE__,__LINE__)
extern void * fcl_realloc_debug(void * p, fcl_size sz, FNLN);
#define fcl_malloc_a(A,B) fcl_malloc_a_debug(A,B,__FILE__,__LINE__)
extern void * fcl_malloc_a_debug(fcl_size sz, fcl_size n, FNLN);
#define fcl_mallocz_a(A,B) fcl_mallocz_a_debug(A,B,__FILE__,__LINE__)
extern void * fcl_mallocz_a_debug(fcl_size sz, fcl_size n, FNLN);
#define fcl_realloc_a(A,B,C) fcl_realloc_a_debug(A,B,C,__FILE__,__LINE__)
extern void * fcl_realloc_a_debug(void * p, fcl_size sz, fcl_size n, FNLN);
#define fcl_free(A) fcl_free_debug(A,__FILE__,__LINE__)
extern void fcl_free_debug(void * p, FNLN);
#define fcl_free_c(A) fcl_free_c_debug(A,__FILE__,__LINE__)
extern void fcl_free_c_debug(void const * p, FNLN);

#define fcl_reallocfree(A,B) fcl_reallocfree_debug(A,B,__FILE__,__LINE__)
extern void * fcl_reallocfree_debug(void * p, fcl_size sz, FNLN);
#define fcl_reallocfree_a(A,B,C) \
  fcl_reallocfree_a_debug(A,B,C,__FILE__,__LINE__)
extern void * fcl_reallocfree_a_debug(void * p, fcl_size sz, fcl_size n, FNLN);

#define fcl_malloc_check(A) fcl_malloc_check_debug(A,__FILE__,__LINE__)
extern void * fcl_malloc_check_debug(fcl_size sz, FNLN);
#define fcl_mallocz_check(A) fcl_mallocz_check_debug(A,__FILE__,__LINE__)
extern void * fcl_mallocz_check_debug(fcl_size sz, FNLN);
#define fcl_realloc_check(A,B) fcl_realloc_check_debug(A,B,__FILE__,__LINE__)
extern void * fcl_realloc_check_debug(void * p, fcl_size sz, FNLN);
#define fcl_malloc_a_check(A,B) fcl_malloc_a_check_debug(A,B,__FILE__,__LINE__)
extern void * fcl_malloc_a_check_debug(fcl_size sz, fcl_size n, FNLN);
#define fcl_mallocz_a_check(A,B) \
  fcl_mallocz_a_check_debug(A,B,__FILE__,__LINE__)
extern void * fcl_mallocz_a_check_debug(fcl_size sz, fcl_size n, FNLN);
#define fcl_realloc_a_check(A,B,C) \
  fcl_realloc_a_check_debug(A,B,C,__FILE__,__LINE__)
extern void * fcl_realloc_a_check_debug
  (void * p, fcl_size sz, fcl_size n, FNLN);
#define fcl_free_check(A) fcl_free_check_debug(A,__FILE__,__LINE__)
extern void fcl_free_check_debug(void * p, FNLN);
#define fcl_free_c_check(A) fcl_free_c_check_debug(A,__FILE__,__LINE__)
extern void fcl_free_c_check_debug(void const * p, FNLN);

#define fcl_strdup(A) fcl_strdup_debug(A,__FILE__,__LINE__)
extern char * fcl_strdup_debug(char const * st, FNLN);
#define fcl_strndup(A,B) fcl_strndup_debug(A,B,__FILE__,__LINE__)
extern char * fcl_strndup_debug(char const * st, fcl_size sz, FNLN);
#define fcl_substrdup(A,B) fcl_substrdup_debug(A,B,__FILE__,__LINE__)
extern char * fcl_substrdup_debug(char const * st, fcl_size sz, FNLN);
#define fcl_memdup(A,B) fcl_memdup_debug(A,B,__FILE__,__LINE__)
extern void * fcl_memdup_debug(void const * p0, fcl_size sz, FNLN);
#define fcl_strdup_check(A) fcl_strdup_check_debug(A,__FILE__,__LINE__)
extern char * fcl_strdup_check_debug(char const * st, FNLN);
#define fcl_strndup_check(A,B) fcl_strndup_check_debug(A,B,__FILE__,__LINE__)
extern char * fcl_strndup_check_debug(char const * st, fcl_size sz, FNLN);
#define fcl_substrdup_check(A,B) fcl_substrdup_check_debug(A,B,__FILE__,__LINE__)
extern char * fcl_substrdup_check_debug(char const * st, fcl_size sz, FNLN);
#define fcl_memdup_check(A,B) fcl_memdup_check_debug(A,B,__FILE__,__LINE__)
extern void * fcl_memdup_check_debug(void const * p0, fcl_size sz, FNLN);

#undef FNLN

#endif

