/*
 * This file is part of firk's Common C Library
 *  qprint.h - qlog-compatible formatted string printing
 *
 * (c) firk 2021; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef FCL_INCLUDED_QPRINT_H
#define FCL_INCLUDED_QPRINT_H

typedef struct {
  int fd;           /* fd to write() to; -1 = only write to buffer */
  int fail;         /* non-zero = stop printing */
  char * buf;       /* buffer, NULL=unbuffered output */
  size_t bufsize;   /* size ofbuffer, 0=unbuffered output */
  size_t bufpos;    /* number of currently stored bytes in buffer */
  size_t rl, ml, mlp; /* for internal temporary use */
} qprint_ctx;

extern qprint_ctx qp_out, qp_err;

extern void qp_init_buf(qprint_ctx * ctx, char * dest, size_t destsz);
extern int qp_flush(qprint_ctx * ctx);
extern int qp_printf(qprint_ctx * ctx, char const * fmt, ...);
extern int qp_snprintf(char * dest, size_t destsize, char const * fmt, ...);
#ifdef FCL_WITH_VA_LIST
extern int qp_vprintf(qprint_ctx * ctx, char const * fmt, va_list arg);
extern int qp_vsnprintf(char * d, size_t dsz, char const * fmt, va_list arg);
#endif
extern int qp_putc(qprint_ctx * ctx, char c);
extern int qp_putce(qprint_ctx * ctx, char c);
extern int qp_puts(qprint_ctx * ctx, char const * s);
extern int qp_putse(qprint_ctx * ctx, char const * s);
extern int qp_putb(qprint_ctx * ctx, char const * s, size_t len);
extern int qp_putbe(qprint_ctx * ctx, char const * s, size_t len);
extern void qp_check(qprint_ctx * ctx);

#define QP_FOPEN_DEFAULT   0x0000 /* create if not exist; append */
#define QP_FOPEN_EXISTING  0x1000 /* only existing; append */
#define QP_FOPEN_NEW       0x2000 /* only new file; append */
#define QP_FOPEN_TRUNC     0x3000 /* create (truncate if exist); append */
#define QP_FOPEN_NOINTR    0x4000 /* fight against EINTR */
#define QP_FOPEN_NOFOLLOW  0x8000 /* do not follow symlinks */

extern int qp_fopen(qprint_ctx * ctx, char const * name, unsigned int mode);
extern int qp_fopenat(qprint_ctx * ctx, int dirfd, char const * name,
                      unsigned int mode);
extern int qp_fclose(qprint_ctx * ctx);

#endif

