/*
 * This file is part of firk's Common C Library
 *  qscan.h - file/stream reader
 *
 * (c) firk 2021-2022; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef FCL_INCLUDED_QSCAN_H
#define FCL_INCLUDED_QSCAN_H

typedef struct {
  /* fd to read from */
  int           fd;
  /* input buffering; NULL = disable */
  char *        buf;
  fcl_size      bufsz;
  fcl_size      buflen;
  fcl_size      bufpos;
  /* target buffer to store string */
  char *        b;
  fcl_size      bmax; /* max size to realloc (used only with QS_ALLOC) */
  fcl_size      bsz;  /* current allocated size */
  fcl_size      bpos; /* current number of stored data bytes */
  uint          flags;
} qscan_ctx;

#define QS_STATIC   0 /* only use preallocated ctx->b */
#define QS_ALLOC    1 /* reallocate ctx->b if needed */
#define QS_NOINTR   2 /* auto try again after EINTR */
#define QS_DOSNL    4 /* strip CR before NL */

/*
  return:
    -1/ENOMEM = ctx->b malloc/realloc failed
    -1/ENOSPC = ctx->bmax (QS_ALLOC) or ctx->bsz (QS_STATIC) limit reached
    -1/EINVAL = wrong ctx data:
      bpos>bsz
      !b || !bsz (QS_STATIC)
      !b && (bpos || bsz || !bmax) (QS_ALLOC)
    -1/errno = error from read() syscall
    0 = read() returned zero (EOF reached)
    1 = done, got newline
  ctx->bpos is number of stored data bytes (not including terminating zero)
  ctx->b[ctx->bpos] is zero when retval >=0, and undefined for retval -1
  newline character itself ('\n') is not stored in buffer
  if QS_DOSNL flag set, CR character ('\r') immediately before NL also stripped
  
  when called again after error or EOF, qscan() will try to continue
  when called again after "done", it will also try to continue, but without
    knowledge about previous newline
 */
extern int qscan(qscan_ctx * ctx);
extern int qs_gets(qscan_ctx * ctx, char * buf, fcl_size maxlen,
                   fcl_size * rlen);

#endif

