/*
 * This file is part of firk's Common C Library
 *  strhash.c - string hashing functions
 *
 * (c) firk 2020; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#include <string.h>
#include "types.h"
#include "strhash.h"

/*********************************************************************/
extern uint32 strhash_djb2(char const * st) {
  uint32 h;
  unsigned char c;
  for(h=5381; c=*st; st++) {
    h += h<<5;
    h ^= c;
  }
  return h;
}

extern uint32 strcasehash_djb2(char const * st) {
  uint32 h;
  unsigned char c;
  for(h=5381; c=*st; st++) {
    if(c>='A' && c<='Z') c |= 32;
    h += h<<5;
    h ^= c;
  }
  return h;
}

extern uint32 memhash_djb2(uchar const * st, size_t n) {
  uint32 h;
  for(h=5381; n--; st++) {
    h += h<<5;
    h ^= *st;
  }
  return h;
}

extern uint32 strhash_djb2_c(char const * st, uint32 h) {
  unsigned char c;
  for( ; c=*st; st++) {
    h += h<<5;
    h ^= c;
  }
  return h;
}

extern uint32 strcasehash_djb2_c(char const * st, uint32 h) {
  unsigned char c;
  for( ; c=*st; st++) {
    if(c>='A' && c<='Z') c |= 32;
    h += h<<5;
    h ^= c;
  }
  return h;
}

extern uint32 memhash_djb2_c(uchar const * st, size_t n, uint32 h) {
  for( ; n--; st++) {
    h += h<<5;
    h ^= *st;
  }
  return h;
}

/*********************************************************************/
extern uint32 strhash(char const * st) {
  uint32 h;
  unsigned char c;
  for(h=648391; c=*st; st++) {
    h *= 1477777711;
    h ^= c;
  }
  return h;
}

extern uint32 strcasehash(char const * st) {
  uint32 h;
  unsigned char c;
  for(h=648391; c=*st; st++) {
    if(c>='A' && c<='Z') c |= 32;
    h *= 1477777711;
    h ^= c;
  }
  return h;
}

extern uint32 memhash(uchar const * st, size_t n) {
  uint32 h;
  for(h=648391; n--; st++) {
    h *= 1477777711;
    h ^= *st;
  }
  return h;
}

extern uint32 strhash_c(char const * st, uint32 h) {
  unsigned char c;
  for( ; c=*st; st++) {
    h *= 1477777711;
    h ^= c;
  }
  return h;
}

extern uint32 strcasehash_c(char const * st, uint32 h) {
  unsigned char c;
  for( ; c=*st; st++) {
    if(c>='A' && c<='Z') c |= 32;
    h *= 1477777711;
    h ^= c;
  }
  return h;
}

extern uint32 memhash_c(uchar const * st, size_t n, uint32 h) {
  for( ; n--; st++) {
    h *= 1477777711;
    h ^= *st;
  }
  return h;
}

