/*
 * This file is part of firk's Cryptography Library
 * aes.h - AES (Rijndael) cipher implementation
 * (c) firk 2015; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef INCLUDED_FCRYPT_AES_H
#define INCLUDED_FCRYPT_AES_H

#define MIN_AES_KEYSIZE   4
#define MAX_AES_KEYSIZE   8
#define MIN_AES_DATASIZE  4
#define MAX_AES_DATASIZE  8
#define MIN_AES_ROUNDS    10
#define MAX_AES_ROUNDS    14
typedef struct _aes_key {
  uint32    ksize; /* in 32-bit words; AES: 4, 6 or 8 */
  uint32    dsize; /* in 32-bit words; AES: 4; original: 4, 6 or 8 */
  uint32    nrounds; /* max(ksize, dsize)+6 */
  uint32    rndkeys[MAX_AES_DATASIZE*(MAX_AES_ROUNDS+1)];
  /* no need to keep original cipher key here */
} aes_key;

extern int aes_prepare_key(aes_key * k, uint32 ksz, uint32 dsz, uint32 * key);
extern void aes_encrypt(aes_key * k, uint32 * data);
extern void aes_decrypt(aes_key * k, uint32 * data);

#endif

