/*
 * This file is part of firk's Cryptography Library
 * aes256.S - AES-256 cipher implementation
 * (c) firk 2015, 2021; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

.intel_syntax noprefix
        .text
        .globl  aes256_prepare_key
        .type   aes256_prepare_key, @function
        .globl  aes256_encrypt
        .type   aes256_encrypt, @function
        .globl  aes256_decrypt
        .type   aes256_decrypt, @function

.balign 16
sbox:
	.byte 0x63, 0x7C, 0x77, 0x7B, 0xF2, 0x6B, 0x6F, 0xC5
	.byte 0x30, 0x01, 0x67, 0x2B, 0xFE, 0xD7, 0xAB, 0x76
	.byte 0xCA, 0x82, 0xC9, 0x7D, 0xFA, 0x59, 0x47, 0xF0
	.byte 0xAD, 0xD4, 0xA2, 0xAF, 0x9C, 0xA4, 0x72, 0xC0
	.byte 0xB7, 0xFD, 0x93, 0x26, 0x36, 0x3F, 0xF7, 0xCC
	.byte 0x34, 0xA5, 0xE5, 0xF1, 0x71, 0xD8, 0x31, 0x15
	.byte 0x04, 0xC7, 0x23, 0xC3, 0x18, 0x96, 0x05, 0x9A
	.byte 0x07, 0x12, 0x80, 0xE2, 0xEB, 0x27, 0xB2, 0x75
	.byte 0x09, 0x83, 0x2C, 0x1A, 0x1B, 0x6E, 0x5A, 0xA0
	.byte 0x52, 0x3B, 0xD6, 0xB3, 0x29, 0xE3, 0x2F, 0x84
	.byte 0x53, 0xD1, 0x00, 0xED, 0x20, 0xFC, 0xB1, 0x5B
	.byte 0x6A, 0xCB, 0xBE, 0x39, 0x4A, 0x4C, 0x58, 0xCF
	.byte 0xD0, 0xEF, 0xAA, 0xFB, 0x43, 0x4D, 0x33, 0x85
	.byte 0x45, 0xF9, 0x02, 0x7F, 0x50, 0x3C, 0x9F, 0xA8
	.byte 0x51, 0xA3, 0x40, 0x8F, 0x92, 0x9D, 0x38, 0xF5
	.byte 0xBC, 0xB6, 0xDA, 0x21, 0x10, 0xFF, 0xF3, 0xD2
	.byte 0xCD, 0x0C, 0x13, 0xEC, 0x5F, 0x97, 0x44, 0x17
	.byte 0xC4, 0xA7, 0x7E, 0x3D, 0x64, 0x5D, 0x19, 0x73
	.byte 0x60, 0x81, 0x4F, 0xDC, 0x22, 0x2A, 0x90, 0x88
	.byte 0x46, 0xEE, 0xB8, 0x14, 0xDE, 0x5E, 0x0B, 0xDB
	.byte 0xE0, 0x32, 0x3A, 0x0A, 0x49, 0x06, 0x24, 0x5C
	.byte 0xC2, 0xD3, 0xAC, 0x62, 0x91, 0x95, 0xE4, 0x79
	.byte 0xE7, 0xC8, 0x37, 0x6D, 0x8D, 0xD5, 0x4E, 0xA9
	.byte 0x6C, 0x56, 0xF4, 0xEA, 0x65, 0x7A, 0xAE, 0x08
	.byte 0xBA, 0x78, 0x25, 0x2E, 0x1C, 0xA6, 0xB4, 0xC6
	.byte 0xE8, 0xDD, 0x74, 0x1F, 0x4B, 0xBD, 0x8B, 0x8A
	.byte 0x70, 0x3E, 0xB5, 0x66, 0x48, 0x03, 0xF6, 0x0E
	.byte 0x61, 0x35, 0x57, 0xB9, 0x86, 0xC1, 0x1D, 0x9E
	.byte 0xE1, 0xF8, 0x98, 0x11, 0x69, 0xD9, 0x8E, 0x94
	.byte 0x9B, 0x1E, 0x87, 0xE9, 0xCE, 0x55, 0x28, 0xDF
	.byte 0x8C, 0xA1, 0x89, 0x0D, 0xBF, 0xE6, 0x42, 0x68
	.byte 0x41, 0x99, 0x2D, 0x0F, 0xB0, 0x54, 0xBB, 0x16

rbox:
	.byte 0x52, 0x09, 0x6A, 0xD5, 0x30, 0x36, 0xA5, 0x38
	.byte 0xBF, 0x40, 0xA3, 0x9E, 0x81, 0xF3, 0xD7, 0xFB
	.byte 0x7C, 0xE3, 0x39, 0x82, 0x9B, 0x2F, 0xFF, 0x87
	.byte 0x34, 0x8E, 0x43, 0x44, 0xC4, 0xDE, 0xE9, 0xCB
	.byte 0x54, 0x7B, 0x94, 0x32, 0xA6, 0xC2, 0x23, 0x3D
	.byte 0xEE, 0x4C, 0x95, 0x0B, 0x42, 0xFA, 0xC3, 0x4E
	.byte 0x08, 0x2E, 0xA1, 0x66, 0x28, 0xD9, 0x24, 0xB2
	.byte 0x76, 0x5B, 0xA2, 0x49, 0x6D, 0x8B, 0xD1, 0x25
	.byte 0x72, 0xF8, 0xF6, 0x64, 0x86, 0x68, 0x98, 0x16
	.byte 0xD4, 0xA4, 0x5C, 0xCC, 0x5D, 0x65, 0xB6, 0x92
	.byte 0x6C, 0x70, 0x48, 0x50, 0xFD, 0xED, 0xB9, 0xDA
	.byte 0x5E, 0x15, 0x46, 0x57, 0xA7, 0x8D, 0x9D, 0x84
	.byte 0x90, 0xD8, 0xAB, 0x00, 0x8C, 0xBC, 0xD3, 0x0A
	.byte 0xF7, 0xE4, 0x58, 0x05, 0xB8, 0xB3, 0x45, 0x06
	.byte 0xD0, 0x2C, 0x1E, 0x8F, 0xCA, 0x3F, 0x0F, 0x02
	.byte 0xC1, 0xAF, 0xBD, 0x03, 0x01, 0x13, 0x8A, 0x6B
	.byte 0x3A, 0x91, 0x11, 0x41, 0x4F, 0x67, 0xDC, 0xEA
	.byte 0x97, 0xF2, 0xCF, 0xCE, 0xF0, 0xB4, 0xE6, 0x73
	.byte 0x96, 0xAC, 0x74, 0x22, 0xE7, 0xAD, 0x35, 0x85
	.byte 0xE2, 0xF9, 0x37, 0xE8, 0x1C, 0x75, 0xDF, 0x6E
	.byte 0x47, 0xF1, 0x1A, 0x71, 0x1D, 0x29, 0xC5, 0x89
	.byte 0x6F, 0xB7, 0x62, 0x0E, 0xAA, 0x18, 0xBE, 0x1B
	.byte 0xFC, 0x56, 0x3E, 0x4B, 0xC6, 0xD2, 0x79, 0x20
	.byte 0x9A, 0xDB, 0xC0, 0xFE, 0x78, 0xCD, 0x5A, 0xF4
	.byte 0x1F, 0xDD, 0xA8, 0x33, 0x88, 0x07, 0xC7, 0x31
	.byte 0xB1, 0x12, 0x10, 0x59, 0x27, 0x80, 0xEC, 0x5F
	.byte 0x60, 0x51, 0x7F, 0xA9, 0x19, 0xB5, 0x4A, 0x0D
	.byte 0x2D, 0xE5, 0x7A, 0x9F, 0x93, 0xC9, 0x9C, 0xEF
	.byte 0xA0, 0xE0, 0x3B, 0x4D, 0xAE, 0x2A, 0xF5, 0xB0
	.byte 0xC8, 0xEB, 0xBB, 0x3C, 0x83, 0x53, 0x99, 0x61
	.byte 0x17, 0x2B, 0x04, 0x7E, 0xBA, 0x77, 0xD6, 0x26
	.byte 0xE1, 0x69, 0x14, 0x63, 0x55, 0x21, 0x0C, 0x7D

/* maximum used index is [(dsz*(nr+1)-1)/ksz]
    ksz     4   4   4   6   6   6   8   8   8
    dsz     4   6   8   4   6   8   4   6   8
    nr      10  12  14  12  12  14  14  14  14
    max_idx 10  19  29  8   12  19  7   11  14 */
rcon:
	.byte 0x00, 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40
	.byte 0x80, 0x1B, 0x36, 0x6C, 0xD8, 0xAB, 0x4D, 0x9A
	.byte 0x2F, 0x5E, 0xBC, 0x63, 0xC6, 0x97, 0x35, 0x6A
	.byte 0xD4, 0xB3, 0x7D, 0xFA, 0xEF, 0xC5, 0x91, 0x39
	.byte 0x72, 0xE4, 0xD3, 0xBD, 0x61, 0xC2, 0x9F, 0x25
	.byte 0x4A, 0x94, 0x33, 0x66, 0xCC, 0x83, 0x1D, 0x3A
	.byte 0x74, 0xE8, 0xCB, 0x8D, 0x01, 0x02, 0x04, 0x08
	.byte 0x10, 0x20, 0x40, 0x80, 0x1B, 0x36, 0x6C, 0xD8

#include "aes_mix.inc"


/* extern int aes256_prepare_key(aes256_key * k, uint32 * key) */
.balign 16
aes256_prepare_key:
	push	ebx
	mov	edx,[esp+8]
	mov	ebx,[esp+12]
	xor	ecx,ecx
.L1:	mov	eax,[ebx+ecx*4]
	mov	[edx+ecx*4],eax
	inc	ecx
	cmp	ecx,8
	jb	.L1
	lea	ebx,sbox
.L2:	mov	eax,[edx+ecx*4-4]
	test	ecx,7
	jz	.L3
	test	ecx,3
	jz	.L4
.L5:	xor	eax,[edx+ecx*4-32]
	mov	[edx+ecx*4],eax
	inc	ecx
	cmp	ecx,60
	jb	.L2
	xor	eax,eax
	pop	ebx
	ret
.L3:	xlat
	rol	eax,8
	xlat
	rol	eax,8
	xlat
	rol	eax,8
	xlat
	shr	ecx,3
	xor	al,rcon[ecx]
	shl	ecx,3
	jmp	.L5
.L4:	xlat
	rol	eax,8
	xlat
	rol	eax,8
	xlat
	rol	eax,8
	xlat
	rol	eax,8
	jmp	.L5


/* assume correct key data and round number */
/* aes_xor(aes256_key * k = EBX, uint32 nround = EDI, uint32 * data = ESI) */
.balign 16
aes_xor:
	mov	edx,edi
	shl	edx,4
	add	edx,ebx
#ifndef AES_USE_SSE_XXX /* SSE slower? */
	mov	eax,[edx]
	xor	[esi],eax
	mov	eax,[edx+4]
	xor	[esi+4],eax
	mov	eax,[edx+8]
	xor	[esi+8],eax
	mov	eax,[edx+12]
	xor	[esi+12],eax
#else
	movdqu	xmm0,[esi]
	pxor	xmm0,[edx]
	movdqu	[esi],xmm0
#endif
	ret

/* static void aes_shift(uint32 * data = ESI) */
.balign 16
aes_shift:
	push	ebx
	mov	eax,[esi]
	mov	ecx,[esi+4]
	mov	edx,[esi+8]
	mov	ebx,[esi+12]
	mov	ah, ch
	mov	ch, dh
	mov	dh, bh
	mov	bh, [esi+1]
	rol	eax, 16
	rol	ecx, 16
	rol	edx, 16
	rol	ebx, 16
	mov	ah, ch
	mov	ch, dh
	mov	dh, bh
	mov	bh, [esi+3]
	xchg	ax, dx
	xchg	cx, bx
	rol	eax, 16
	rol	ecx, 16
	rol	edx, 16
	rol	ebx, 16
	mov	[esi],eax
	mov	[esi+4],ecx
	mov	[esi+8],edx
	mov	[esi+12],ebx
	pop	ebx
	ret

/* static void aes_unshift(uint32 * data = ESI) */
.balign 16
aes_unshift:
	push	ebx
	mov	eax,[esi]
	mov	ecx,[esi+4]
	mov	edx,[esi+8]
	mov	ebx,[esi+12]
	mov	ah, bh
	mov	bh, dh
	mov	dh, ch
	mov	ch, [esi+1]
	rol	eax, 16
	rol	ecx, 16
	rol	edx, 16
	rol	ebx, 16
	mov	ah, bh
	mov	bh, dh
	mov	dh, ch
	mov	ch, [esi+3]
	xchg	ax, dx
	xchg	cx, bx
	rol	eax, 16
	rol	ecx, 16
	rol	edx, 16
	rol	ebx, 16
	mov	[esi],eax
	mov	[esi+4],ecx
	mov	[esi+8],edx
	mov	[esi+12],ebx
	pop	ebx
	ret

#define aes_mix32(P) \
	movzx	ecx,byte ptr [P]    ; \
	mov	eax,aes_mix0[ecx*4] ; \
	movzx	ecx,byte ptr [P+1]  ; \
	xor	eax,aes_mix1[ecx*4] ; \
	movzx	ecx,byte ptr [P+2]  ; \
	xor	eax,aes_mix2[ecx*4] ; \
	movzx	ecx,byte ptr [P+3]  ; \
	xor	eax,aes_mix3[ecx*4] ; \
	mov	[P],eax

#define aes_unmix32(P) \
	movzx	ecx,byte ptr [P]    ; \
	mov	eax,aes_mix4[ecx*4] ; \
	movzx	ecx,byte ptr [P+1]  ; \
	xor	eax,aes_mix5[ecx*4] ; \
	movzx	ecx,byte ptr [P+2]  ; \
	xor	eax,aes_mix6[ecx*4] ; \
	movzx	ecx,byte ptr [P+3]  ; \
	xor	eax,aes_mix7[ecx*4] ; \
	mov	[P],eax

/* static void aes_mix(uint32 * data = ESI) */
.balign 16
aes_mix:
	aes_mix32(esi)
	aes_mix32(esi+4)
	aes_mix32(esi+8)
	aes_mix32(esi+12)
	ret

/* static void aes_unmix(uint32 * data = ESI) */
.balign 16
aes_unmix:
	aes_unmix32(esi)
	aes_unmix32(esi+4)
	aes_unmix32(esi+8)
	aes_unmix32(esi+12)
	ret

#ifdef AES_USE_XLAT
#define XLAT1(P) push ebx ; lea ebx,P
#define XLAT(X) mov al,X ; xlat ; mov X,al
#define XLAT2() pop ebx
#else
#define XLAT1(P) lea ecx,P
#define XLAT(X) movzx eax,byte ptr X ; mov al,[ecx+eax] ; mov X,al
#define XLAT2()
#endif
#define XLAT_BOX() ; \
  XLAT([esi]);XLAT([esi+1]);XLAT([esi+2]);XLAT([esi+3]); \
  XLAT([esi+4]);XLAT([esi+5]);XLAT([esi+6]);XLAT([esi+7]); \
  XLAT([esi+8]);XLAT([esi+9]);XLAT([esi+10]);XLAT([esi+11]); \
  XLAT([esi+12]);XLAT([esi+13]);XLAT([esi+14]);XLAT([esi+15])

#ifndef INLINE_SBOX
#define SBOX_TRANSFORM XLAT1(sbox) ; call sbox_transform ; XLAT2()
.balign 16
sbox_transform: XLAT_BOX() ; ret
#else
#define SBOX_TRANSFORM XLAT1(sbox) ; XLAT_BOX() ; XLAT2()
#endif

/* extern void aes256_encrypt(aes256_key * k, uint32 * data) */
.balign 16
aes256_encrypt:
	push	esi
	push	edi
	push	ebx
#ifndef AES_NEW_ENC
	mov	esi,[esp+20]
	mov	ebx,[esp+16]
	xor	edi,edi
.L1e:	call	aes_xor
	SBOX_TRANSFORM
	call	aes_shift
	cmp	edi,13
	je	.L2e
	call	aes_mix
	inc	edi
	jmp	.L1e
.L2e:	mov	edi,14
	call	aes_xor
#else /* this looks optimized but it slower :( */
	mov	esi,[esp+20]
	mov	ebx,[esp+16]
	xor	edi,edi
	jmp	.L1e
.balign 8
.L10:	call	aes_mix
.L1e:	call	aes_xor
	SBOX_TRANSFORM
	call	aes_shift
	inc	edi
	cmp	edi,14
	jne	.L10
	call	aes_xor
#endif
	pop	ebx
	pop	edi
	pop	esi
	ret
/*
  for(i=0; i<14; i++) {
    aes_xor(k, i, data);
    sbox_transform(data, 16);
    aes_shift(data);
    if(i!=13) aes_mix(data);
  }
  aes_xor(k, 14, data);
*/

#ifndef INLINE_RBOX
#define RBOX_TRANSFORM XLAT1(rbox) ; call rbox_transform ; XLAT2()
.balign 16
rbox_transform: XLAT_BOX() ; ret
#else
#define RBOX_TRANSFORM XLAT1(rbox) ; XLAT_BOX() ; XLAT2()
#endif


/* extern void aes256_decrypt(aes256_key * k, uint32 * data) */
.balign 16
aes256_decrypt:
	push	esi
	push	edi
	push	ebx
	mov	esi,[esp+20]
	mov	ebx,[esp+16]
	mov	edi,14

	call	aes_xor
	call	aes_unshift
	RBOX_TRANSFORM
	dec	edi

.L1d:	call	aes_xor
	call	aes_unmix
	call	aes_unshift
	RBOX_TRANSFORM
	dec	edi
	jnz	.L1d

	call	aes_xor
	pop	ebx
	pop	edi
	pop	esi
	ret

/*
  for(i=14; i>0; i--) {
    aes_xor(k, i, data);
    if(i!=14) aes_unmix(data);
    aes_unshift(data);
    rbox_transform(data, 16);
  }
  aes_xor(k, 0, data);
*/


