/* AES S-box generator */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "../../fcl/types.h"

uchar rev[256], fwd[256], inv[256];

uchar ROR8(uint a, uchar b) {
  uchar c;
  c = (uchar)a;
  c = (c>>b) | (c<<(8-b));
  return c;
}

uint gf28mul(uint a, uint b) {
  uint res;
  if(a>0xFF || b>0xFF) { printf("err\n"); exit(-1); }
  res = 0;
  while(b) {
    if(b&1) res ^= a;
    b >>= 1;
    a <<= 1;
  }
  /* ASSUME a, b was bytes */
  a = 0x10000;
  b = 0x11B00;
  while(a>0xFF) {
    if(res&a) res ^= b;
    a>>=1; b>>=1;
  }
  return res;
}

int main(void) {
  uint a, b;
  bzero(fwd, 256); bzero(inv, 256);
  for(a=0; a<256; a++) {
    for(b=0; b<256; b++) {
      if(gf28mul(a,b)==1) {
        rev[a] = b; rev[b] = a;
      }
    }
  }
  for(a=0; a<256; a++) {
    b = rev[a];
    b = b ^ ROR8(b,4) ^ ROR8(b,5) ^ ROR8(b,6) ^ ROR8(b,7) ^ 0x63;
    fwd[a] = b;
    inv[b] = a;
  }
/*  printf("static uchar u8_rev[256] = {\n");
  for(a=0; a<256; a++) {
    printf("%02X%c%c", rev[a], (a<255)?',':'}', (a%16==15)?'\n':' ');
  }*/
  printf("static uchar sbox[256] = {\n");
  for(a=0; a<256; a++) {
    if(!(a%8)) printf("  ");
    printf("0x%02X", fwd[a]);
    if(a==255) { printf("\n};\n\n"); break; }
    if(a%8==7) printf(",\n"); else printf(", ");
  }
  printf("static uchar rbox[256] = {\n");
  for(a=0; a<256; a++) {
    if(!(a%8)) printf("  ");
    printf("0x%02X", inv[a]);
    if(a==255) { printf("\n};\n\n"); break; }
    if(a%8==7) printf(",\n"); else printf(", ");
  }
  printf("static uint32 rcon[64] = {\n  0x00, ");
  b = 1;
  for(a=1; a<64; a++) {
    if(!(a%8)) printf("  ");
    printf("0x%02X", b);
    if(a==63) { printf("\n};\n\n"); break; }
    if(a%8==7) printf(",\n"); else printf(", ");
    b = gf28mul(b, 2);
  }
  return 0;
}
