/* constants generator for SHA-2 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "../fcl/types.h"

int is_prime(ushort x) {
  ushort y;
  if(x==0) return 0;
  if(x<4) return 1;
  for(y=2;y<x;y++) if(!(x%y)) return 0;
  return 1;
}

typedef struct _uint256 { uint32 x[8]; } uint256;
void mul256(uint256 * a, uint256 * b, uint256 * res) {
  uint64 xa, xb;
  uint32 p1, p2;
  int i, j, k;
  memset(res, 0, sizeof(uint256)); 
  for(i=0; i<8; i++) for(j=0; j<8; j++) {
    xa = a->x[i]; xb = b->x[j];
    xa *= xb;
    p1 = (uint32)(xa);
    p2 = (uint32)(xa>>32);
    k = i+j;
    while(k<8) {
      xa = res->x[k];
      xa += p1;
      res->x[k] = (uint32)xa;
      p1 = (uint32)(xa>>32);
      k++;
    }
    k = i+j+1;
    while(k<8) {
      xa = res->x[k];
      xa += p2;
      res->x[k] = (uint32)xa;
      p2 = (uint32)(xa>>32);
      k++;
    }
  }
}


void cuberoot(uint64 val, uint64 * res) {
  uint64 q, pmin, pmax, p;
  uint256 a,b,c,d;
  for(q=1;q<val;q++) if(q*q*q>val) break;
  if(q*q*q<=val) { fprintf(stderr,"err234324\n"); exit(-1); }
  q--;
  if(q*q*q>val) { fprintf(stderr,"err843255\n"); exit(-1); }
  pmin = 0;
  pmax = (ullong)-1;
  memset(&a, 0, sizeof(a));
  a.x[2] = (uint32)q;
  while(pmin<pmax) {
    p = pmax-(pmax-pmin)/2;
    a.x[0] = (uint32)(p);
    a.x[1] = (uint32)(p>>32);
    mul256(&a, &a, &b);
    mul256(&a, &b, &c);
    if(c.x[6]<val) { pmin=p; continue; }
    if(c.x[6]>val) { pmax=p-1; continue; }
    pmax=p-1; /* valid for primes, invalid in common case */
  }
  *res = pmin;
}

int main(void) {
  ushort x, n;
  uint64 res;
  n = 0;
  for(x=2;x<=409;x++) if(is_prime(x)) {
    cuberoot(x, &res);
    printf("0x%016llXULL", (ullong)res);
    printf(",");
    n++;
    if(n%3) printf(" "); else printf("~");
  }
}

