/*
 * This file is part of firk's Cryptography Library
 * hmac.h - rfc2104 HMAC implementation
 * (c) firk 2020; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef INCLUDED_FCRYPT_HMAC_H
#define INCLUDED_FCRYPT_HMAC_H

#ifdef INCLUDED_FCRYPT_MD5_H
typedef struct { uint32 istate[4], ostate[4]; } hmac_md5_key;
extern void hmac_md5_initkey(hmac_md5_key * k,
                              uint8 const * key, size_t keylen);
extern void hmac_md5_init(md5_ctx * ctx, hmac_md5_key const * k);
extern void hmac_md5_final(md5_ctx * ctx, hmac_md5_key const * k,
                            uint8 * hash);
extern void hmac_md5_hash(uint8 * hash, hmac_md5_key const * k,
                           char const * data, size_t len);
#endif

#ifdef INCLUDED_FCRYPT_SHA1_H
typedef struct { uint32 istate[5], ostate[5]; } hmac_sha1_key;
extern void hmac_sha1_initkey(hmac_sha1_key * k,
                              uint8 const * key, size_t keylen);
extern void hmac_sha1_init(sha1_ctx * ctx, hmac_sha1_key const * k);
extern void hmac_sha1_final(sha1_ctx * ctx, hmac_sha1_key const * k,
                            uint8 * hash);
extern void hmac_sha1_hash(uint8 * hash, hmac_sha1_key const * k,
                           char const * data, size_t len);
#endif

#ifdef INCLUDED_FCRYPT_SHA256_H
typedef struct { uint32 istate[8], ostate[8]; } hmac_sha256_key;
extern void hmac_sha256_initkey(hmac_sha256_key * k,
                              uint8 const * key, size_t keylen);
extern void hmac_sha256_init(sha256_ctx * ctx, hmac_sha256_key const * k);
extern void hmac_sha256_final(sha256_ctx * ctx, hmac_sha256_key const * k,
                            uint8 * hash);
extern void hmac_sha256_hash(uint8 * hash, hmac_sha256_key const * k,
                           char const * data, size_t len);
#endif

#ifdef INCLUDED_FCRYPT_SHA512_H
#ifdef HAVE_INT64
typedef struct { uint64 istate[8], ostate[8]; } hmac_sha512_key;
extern void hmac_sha512_initkey(hmac_sha512_key * k,
                              uint8 const * key, size_t keylen);
extern void hmac_sha512_init(sha512_ctx * ctx, hmac_sha512_key const * k);
extern void hmac_sha512_final(sha512_ctx * ctx, hmac_sha512_key const * k,
                            uint8 * hash);
extern void hmac_sha512_hash(uint8 * hash, hmac_sha512_key const * k,
                           char const * data, size_t len);
#endif
#endif

#endif

