/*
 * This file is part of firk's Cryptography Library
 * kuznyechik.h - GOST R 34.12-2015 128-bit (Kuznyechik) cipher implementation
 * (c) firk 2021; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef INCLUDED_FCRYPT_KUZNYECHIK_H
#define INCLUDED_FCRYPT_KUZNYECHIK_H

typedef struct _kuz_key {
  uint8     rndkeys[320]; /* sse2-optimized uses x2 space */
} kuz_key;

extern void kuz_prepare_key(kuz_key * k, uint8 const * key);
extern void kuz_encrypt(kuz_key const * k, uint8 * data);
extern void kuz_decrypt(kuz_key const * k, uint8 * data);
extern void kuz_encrypt_aligned(kuz_key const * k, uint8 * data);
extern void kuz_decrypt_aligned(kuz_key const * k, uint8 * data);

#endif

