/*
 * This file is part of firk's Cryptography Library
 * (c) firk 2008, 2011, 2015; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#include <sys/types.h>
#include "rc4.h"

extern void RC4_makekey(RC4Key * key, unsigned char const * src,
                        size_t srclen, unsigned int droplen){
  unsigned int a, s;
  unsigned char b, c;
  if(srclen<0x100) s=srclen; else s=0x100;
  for(a=0;a<0x100;a++) key->data[a]=a;
  key->i=key->j=0;
  for(a=0,b=0;a<0x100;a++){
    b+=src[a % s]+key->data[a];
    c=key->data[a];key->data[a]=key->data[b];key->data[b]=c;
  }
  if(droplen>0) RC4_keydrop(key, droplen);
}

extern void RC4_keydrop(RC4Key * key, unsigned int droplen){
  unsigned char a, b, c;
  while(droplen--) {
    a = ++key->i;
    key->j += key->data[a];
    b = key->j;
    c=key->data[a];key->data[a]=key->data[b];key->data[b]=c;
  }
}

extern void RC4_xorcrypt(RC4Key * key, unsigned char * data, size_t len){
  unsigned char a, b, c;
  while(len--) {
    a = ++key->i;
    key->j += key->data[a];
    b = key->j;
    c=key->data[a];key->data[a]=key->data[b];key->data[b]=c;
    (*data)^=key->data[(key->data[key->i]+key->data[key->j])&0xFF];
    data++;
  }
}

