#!/bin/sh -e

SALSA20="$1"
BENCH="$2"
# asm_ia looks optimal for non-sse2
# asm_sse2_a looks optimal for sse2

if [ "$SALSA20" = "" ]; then
  CCI -O1 -I. salsa20/salsa20.c -o salsa20.o
elif [ $SALSA20 = "asm" ]; then
  CCI salsa20/salsa20.S -o salsa20.o
elif [ $SALSA20 = "asm_a" ]; then
  CCI -DALIGN_ROUND4 salsa20/salsa20.S -o salsa20.o
elif [ $SALSA20 = "asm_i" ]; then
  CCI -DINLINE_ROUND4 salsa20/salsa20.S -o salsa20.o
elif [ $SALSA20 = "asm_ia" ]; then
  CCI -DINLINE_ROUND4 -DALIGN_ROUND16 salsa20/salsa20.S -o salsa20.o
elif [ $SALSA20 = "asm_ii" ]; then
  CCI -DINLINE_ROUND4 -DINLINE_ROUND16 salsa20/salsa20.S -o salsa20.o
elif [ $SALSA20 = "asm_sse2" ]; then
  CCI salsa20/salsa20_sse2.S -o salsa20.o
elif [ $SALSA20 = "asm_sse2_a" ]; then
  CCI -DALIGN_ROUND16 salsa20/salsa20_sse2.S -o salsa20.o
elif [ $SALSA20 = "asm_sse2_i" ]; then
  CCI -DINLINE_ROUND16 salsa20/salsa20_sse2.S -o salsa20.o
else
  echo "unknown salsa20 compile mode"
  exit 1
fi

if [ -n "$BENCH" ]; then
  if ( uname -a | grep Linux >> /dev/null ) ; then
    TIME="time -f real=%e;user=%U;sys=%S"
  else
    TIME="time"
  fi
  CCI salsa20/bench.c
  LDI bench salsa20.o bench.o
  rm salsa20.o bench.o
  echo "salsa20 $SALSA20 $BENCH MBytes"
  $TIME ./bench 0 "$BENCH"
  echo "chacha20 $SALSA20 $BENCH MBytes"
  $TIME ./bench 1 "$BENCH"
  echo "Done"
  rm bench
fi

exit 0
