/*
 * This file is part of firk's Cryptography Library
 * salsa20_sse2.S - Salsa20 and ChaCha20 stream cipher implementation w/SSE2
 * (c) firk 2015, 2021; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

.intel_syntax noprefix
        .text
        .globl  salsa20_genrandom512
        .type   salsa20_genrandom512, @function
        .globl  chacha20_genrandom512
        .type   chacha20_genrandom512, @function

/* NOTE: this is hardcoded in _step() functions */
#define A eax
#define B ecx
#define C edx
#define D esi
#define TMP edi
#ifdef INLINE_ROUND16
#define DATA(i) [esp+(i*4)]
#else
#define DATA(i) [esp+4+(i*4)]
#endif



/*
static void round4_salsa(uint32 * d0, uint32 * d1, uint32 * d2, uint32 * d3) {
  uint32 a, b, c, d;
  a=*d0;b=*d1;c=*d2;d=*d3;
  b ^= ROL32(a+d, 7);
  c ^= ROL32(b+a, 9);
  d ^= ROL32(c+b, 13);
  a ^= ROL32(d+c, 18);
  *d0=a;*d1=b;*d2=c;*d3=d;
}
*/
#define round4_salsa_body  \
	lea	TMP, [A+D] ; \
	rol	TMP, 7     ; \
	xor	B, TMP     ; \
	lea	TMP, [A+B] ; \
	rol	TMP, 9     ; \
	xor	C, TMP     ; \
	lea	TMP, [B+C] ; \
	rol	TMP, 13    ; \
	xor	D, TMP     ; \
	lea	TMP, [C+D] ; \
	rol	TMP, 18    ; \
	xor	A, TMP

/*
static void round16_salsa(uint32 * d) {
  round4_salsa(d+0, d+4, d+8, d+12);
  round4_salsa(d+5, d+9, d+13, d+1);
  round4_salsa(d+10, d+14, d+2, d+6);
  round4_salsa(d+15, d+3, d+7, d+11);
  round4_salsa(d+0, d+1, d+2, d+3);
  round4_salsa(d+5, d+6, d+7, d+4);
  round4_salsa(d+10, d+11, d+8, d+9);
  round4_salsa(d+15, d+12, d+13, d+14);
fields reordered for easy SSE2
tN - traditional position
N - reordered position
  t0 = 0, t4 = 4, t8 = 8, t12 = 12
  t5 = 1, t9 = 5, t13 = 9, t1 = 13
  t10 = 2, t14 = 6, t2 = 10, t6 = 14
  t15 = 3, t3 = 7, t7 = 11, t11 = 15
}
*/
#define round16_salsa_body \
	round4_salsa_call(0,4,8,12)   ; \
	round4_salsa_call(1,5,9,13)   ; \
	round4_salsa_call(2,6,10,14)  ; \
	round4_salsa_call(3,7,11,15)  ; \
	round4_salsa_call(0,13,10,7)  ; \
	round4_salsa_call(1,14,11,4)  ; \
	round4_salsa_call(2,15,8,5)   ; \
	round4_salsa_call(3,12,9,6)





#ifndef INLINE_ROUND4
#ifdef ALIGN_ROUND4
.balign 16
#endif
round4_salsa:
	round4_salsa_body
	ret
#define round4_salsa_call(iA,iB,iC,iD) \
	mov	A,DATA(iA) ; \
	mov	B,DATA(iB) ; \
	mov	C,DATA(iC) ; \
	mov	D,DATA(iD) ; \
	call	round4_salsa ; \
	mov	DATA(iA),A ; \
	mov	DATA(iB),B ; \
	mov	DATA(iC),C ; \
	mov	DATA(iD),D
#else
#define round4_salsa_call(iA,iB,iC,iD) \
	mov	A,DATA(iA) ; \
	mov	B,DATA(iB) ; \
	mov	C,DATA(iC) ; \
	mov	D,DATA(iD) ; \
	round4_salsa_body  ; \
	mov	DATA(iA),A ; \
	mov	DATA(iB),B ; \
	mov	DATA(iC),C ; \
	mov	DATA(iD),D
#endif

#ifndef INLINE_ROUND16
#ifdef ALIGN_ROUND16
.balign 16
#endif
round16_salsa:
	round16_salsa_body
	ret
#define round16_salsa_call() call round16_salsa
#else
#define round16_salsa_call() round16_salsa_body
#endif

/*
extern void salsa20_step(uint32 * d) {
  uint32 tmp[16];
  uint32 i;
  for(i=0;i<16;i++) tmp[i]=d[i];
  for(i=0;i<10;i++) round16_salsa(tmp);
  for(i=0;i<16;i++) d[i]+=tmp[i];
}
*/
salsa20_step:
	push	esi
	push	edi
	push	ebp
	mov	ebp,esp
	and	esp,0xFFFFFFC0
	sub	esp,0x40
	mov	esi, [ebp+0x10]
	mov	edi, esp
	mov	ecx, 16
	rep	movsd
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	mov	edi, [ebp+0x10]
#define ESP_TO_EDI_32(offs) mov eax,[esp+offs] ; add [edi+offs],eax
#define ESP_TO_EDI_64(offs) ESP_TO_EDI_32(offs) ; ESP_TO_EDI_32(offs+4)
#define ESP_TO_EDI_128(offs) ESP_TO_EDI_64(offs) ; ESP_TO_EDI_64(offs+8)
#define ESP_TO_EDI_256(offs) ESP_TO_EDI_128(offs) ; ESP_TO_EDI_128(offs+16)
#define ESP_TO_EDI_512(offs) ESP_TO_EDI_256(offs) ; ESP_TO_EDI_256(offs+32)
	ESP_TO_EDI_512(0)
	mov	esp,ebp
	pop	ebp
	pop	edi
	pop	esi
	ret

/*
static void round4_chacha(uint32 * d0, uint32 * d1, uint32 * d2, uint32 * d3) {
  uint32 a, b, c, d;
  a=*d0;b=*d1;c=*d2;d=*d3;
  a+=b; d=ROL32(d^a,16);
  c+=d; b=ROL32(b^c,12);
  a+=b; d=ROL32(d^a,8);
  c+=d; b=ROL32(b^c,7);
  *d0=a;*d1=b;*d2=c;*d3=d;
}
*/
#define round4_chacha_body \
	add	A, B  ; \
	xor	D, A  ; \
	rol	D, 16 ; \
	add	C, D  ; \
	xor	B, C  ; \
	rol	B, 12 ; \
	add	A, B  ; \
	xor	D, A  ; \
	rol	D, 8  ; \
	add	C, D  ; \
	xor	B, C  ; \
	rol	B, 7

/*
static void round16_chacha(uint32 * d) {
  round4_chacha(d+0, d+4, d+8, d+12);
  round4_chacha(d+1, d+5, d+9, d+13);
  round4_chacha(d+2, d+6, d+10, d+14);
  round4_chacha(d+3, d+7, d+11, d+15);
  round4_chacha(d+0, d+5, d+10, d+15);
  round4_chacha(d+1, d+6, d+11, d+12);
  round4_chacha(d+2, d+7, d+8, d+13);
  round4_chacha(d+3, d+4, d+9, d+14);
}
*/
#define round16_chacha_body \
	round4_chacha_call(0,4,8,12)  ; \
	round4_chacha_call(1,5,9,13)  ; \
	round4_chacha_call(2,6,10,14) ; \
	round4_chacha_call(3,7,11,15) ; \
	round4_chacha_call(0,5,10,15) ; \
	round4_chacha_call(1,6,11,12) ; \
	round4_chacha_call(2,7,8,13)  ; \
	round4_chacha_call(3,4,9,14)


#ifndef INLINE_ROUND4
#ifdef ALIGN_ROUND4
.balign 16
#endif
round4_chacha:
	round4_chacha_body
	ret
#define round4_chacha_call(iA,iB,iC,iD) \
	mov	A,DATA(iA) ; \
	mov	B,DATA(iB) ; \
	mov	C,DATA(iC) ; \
	mov	D,DATA(iD) ; \
	call	round4_chacha ; \
	mov	DATA(iA),A ; \
	mov	DATA(iB),B ; \
	mov	DATA(iC),C ; \
	mov	DATA(iD),D
#else
#define round4_chacha_call(iA,iB,iC,iD) \
	mov	A,DATA(iA) ; \
	mov	B,DATA(iB) ; \
	mov	C,DATA(iC) ; \
	mov	D,DATA(iD) ; \
	round4_chacha_body  ; \
	mov	DATA(iA),A ; \
	mov	DATA(iB),B ; \
	mov	DATA(iC),C ; \
	mov	DATA(iD),D
#endif

#ifndef INLINE_ROUND16
#ifdef ALIGN_ROUND16
.balign 16
#endif
round16_chacha:
	round16_chacha_body
	ret
#define round16_chacha_call() call round16_chacha
#else
#define round16_chacha_call() round16_chacha_body
#endif


/*
extern void chacha20_step(uint32 * d) {
  uint32 tmp[16];
  uint32 i;
  for(i=0;i<16;i++) tmp[i]=d[i];
  for(i=0;i<10;i++) round16_chacha(tmp);
  for(i=0;i<16;i++) d[i]+=tmp[i];
}
*/
chacha20_step:
	push	esi
	push	edi
	push	ebp
	mov	ebp,esp
	and	esp,0xFFFFFFC0
	sub	esp,0x40
	mov	esi, [ebp+0x10]
	mov	edi, esp
	mov	ecx, 16
	rep	movsd
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	mov	edi, [ebp+0x10]
	ESP_TO_EDI_512(0)
	mov	esp,ebp
	pop	ebp
	pop	edi
	pop	esi
	ret



#undef DATA
#define DATA(i) dword ptr [ecx+(i*4)]

salsa20_genrandom512:
	push	ebx
	mov	ecx,[esp+8]
	mov	edx,[esp+12]
	mov	ebx,[esp+16]
	mov	eax, [edx+0]
	mov	DATA(0), 0x61707865
	mov	DATA(1), 0x3320646E
	mov	DATA(2), 0x79622D32
	mov	DATA(3), 0x6B206574
	mov	DATA(13), eax
	mov	eax, [edx+4]
	mov	DATA(10), eax
	mov	eax, [edx+8]
	mov	DATA(7), eax
	mov	eax, [edx+12]
	mov	DATA(4), eax
	mov	eax, [ebx+0]
	mov	DATA(14), eax
	mov	eax, [ebx+4]
	mov	DATA(11), eax
	mov	ebx, [esp+20]
	mov	eax, [ebx+0]
	mov	DATA(8), eax
	mov	eax, [ebx+4]
	mov	DATA(5), eax
	mov	eax, [edx+16]
	mov	DATA(15), eax
	mov	eax, [edx+20]
	mov	DATA(12), eax
	mov	eax, [edx+24]
	mov	DATA(9), eax
	mov	eax, [edx+28]
	mov	DATA(6), eax
	pop	ebx
	push	ecx
	call	salsa20_step
	pop	ecx
	mov	eax, DATA(1)
	xchg	eax, DATA(5)
	xchg	eax, DATA(9)
	xchg	eax, DATA(13)
	mov	DATA(1), eax
	mov	eax, DATA(2)
	xchg	eax, DATA(10)
	mov	DATA(2), eax
	mov	eax, DATA(6)
	xchg	eax, DATA(14)
	mov	DATA(6), eax
	mov	eax, DATA(3)
	xchg	eax, DATA(15)
	xchg	eax, DATA(11)
	xchg	eax, DATA(7)
	mov	DATA(3), eax
	ret

chacha20_genrandom512:
	push	ebx
	mov	ecx,[esp+8]
	mov	edx,[esp+12]
	mov	ebx,[esp+20]
	mov	DATA(0), 0x61707865
	mov	DATA(1), 0x3320646E
	mov	DATA(2), 0x79622D32
	mov	DATA(3), 0x6B206574
	mov	eax, [edx+0]
	mov	DATA(4), eax
	mov	eax, [edx+4]
	mov	DATA(5), eax
	mov	eax, [edx+8]
	mov	DATA(6), eax
	mov	eax, [edx+12]
	mov	DATA(7), eax
	mov	eax, [edx+16]
	mov	DATA(8), eax
	mov	eax, [edx+20]
	mov	DATA(9), eax
	mov	eax, [edx+24]
	mov	DATA(10), eax
	mov	eax, [edx+28]
	mov	DATA(11), eax
	mov	eax, [ebx+0]
	mov	DATA(12), eax
	mov	eax, [ebx+4]
	mov	DATA(13), eax
	mov	ebx, [esp+16]
	mov	eax, [ebx+0]
	mov	DATA(14), eax
	mov	eax, [ebx+4]
	mov	DATA(15), eax
	pop	ebx
	jmp	chacha20_step

