/*
 * This file is part of firk's Cryptography Library
 * salsa20_sse2.S - Salsa20 and ChaCha20 stream cipher implementation w/SSE2
 * (c) firk 2015, 2021; firk@cantconnect.ru
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

.intel_syntax noprefix
        .text
        .globl  salsa20_genrandom512
        .type   salsa20_genrandom512, @function
        .globl  chacha20_genrandom512
        .type   chacha20_genrandom512, @function

/* NOTE: this is hardcoded in _step() functions */
#define TMP xmm4
#define ROLTMP xmm5

#define ADD32(DEST, A1, A2) \
	movdqa	DEST, A1      ; \
	paddd	DEST, A2

#define ROL32(R,COUNT) \
	movdqa	ROLTMP, R     ; \
	pslld	ROLTMP, COUNT ; \
	psrld	R, 32-COUNT   ; \
	pxor	R, ROLTMP

/* byte-rotate, not bit! */
#define BRL128(R,NBYTE) \
	movdqa	ROLTMP, R     ; \
	pslldq	ROLTMP, NBYTE ; \
	psrldq	R, 16-NBYTE   ; \
	pxor	R, ROLTMP

/*
static void round4_salsa(uint32 * d0, uint32 * d1, uint32 * d2, uint32 * d3) {
  uint32 a, b, c, d;
  a=*d0;b=*d1;c=*d2;d=*d3;
  b ^= ROL32(a+d, 7);
  c ^= ROL32(b+a, 9);
  d ^= ROL32(c+b, 13);
  a ^= ROL32(d+c, 18);
  *d0=a;*d1=b;*d2=c;*d3=d;
}
*/
#define round4_salsa_body(A,B,C,D)  \
	ADD32(	TMP, A, D) ; \
	ROL32(	TMP, 7)    ; \
	pxor	B, TMP     ; \
	ADD32(	TMP, A, B) ; \
	ROL32(	TMP, 9)    ; \
	pxor	C, TMP     ; \
	ADD32(	TMP, B, C) ; \
	ROL32(	TMP, 13)   ; \
	pxor	D, TMP     ; \
	ADD32(	TMP, C, D) ; \
	ROL32(	TMP, 18)   ; \
	pxor	A, TMP

/*
static void round16_salsa(uint32 * d) {
  round4_salsa(d+0, d+4, d+8, d+12);
  round4_salsa(d+5, d+9, d+13, d+1);
  round4_salsa(d+10, d+14, d+2, d+6);
  round4_salsa(d+15, d+3, d+7, d+11);
  round4_salsa(d+0, d+1, d+2, d+3);
  round4_salsa(d+5, d+6, d+7, d+4);
  round4_salsa(d+10, d+11, d+8, d+9);
  round4_salsa(d+15, d+12, d+13, d+14);
fields reordered for easy SSE2
tN - traditional position
N - reordered position
  t0 = 0, t4 = 4, t8 = 8, t12 = 12
  t5 = 1, t9 = 5, t13 = 9, t1 = 13
  t10 = 2, t14 = 6, t2 = 10, t6 = 14
  t15 = 3, t3 = 7, t7 = 11, t11 = 15
}
*/
#define round16_salsa_body \
	round4_salsa_body(xmm0,xmm1,xmm2,xmm3)      ; \
	BRL128(xmm1,4)       ; \
	BRL128(xmm2,8)       ; \
	BRL128(xmm3,12)      ; \
	round4_salsa_body(xmm0,xmm3,xmm2,xmm1)      ; \
	BRL128(xmm1,12)      ; \
	BRL128(xmm2,8)      ; \
	BRL128(xmm3,4)


#ifndef INLINE_ROUND16
#ifdef ALIGN_ROUND16
.balign 16
#endif
round16_salsa:
	round16_salsa_body
	ret
#define round16_salsa_call() call round16_salsa
#else
#define round16_salsa_call() round16_salsa_body
#endif

/*
extern void salsa20_step(uint32 * d) {
  uint32 tmp[16];
  uint32 i;
  for(i=0;i<16;i++) tmp[i]=d[i];
  for(i=0;i<10;i++) round16_salsa(tmp);
  for(i=0;i<16;i++) d[i]+=tmp[i];
}
*/
salsa20_step:
	mov	eax, [esp+4]
	movdqu	xmm0, [eax+0]
	movdqu	xmm1, [eax+16]
	movdqu	xmm2, [eax+32]
	movdqu	xmm3, [eax+48]
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	round16_salsa_call()
	movdqu	xmm4, [eax+0]
	movdqu	xmm5, [eax+16]
	movdqu	xmm6, [eax+32]
	movdqu	xmm7, [eax+48]
	paddd xmm0, xmm4
	paddd xmm1, xmm5
	paddd xmm2, xmm6
	paddd xmm3, xmm7
	movdqu	[eax+0], xmm0
	movdqu	[eax+16], xmm1
	movdqu	[eax+32], xmm2
	movdqu	[eax+48], xmm3
	ret

/*
static void round4_chacha(uint32 * d0, uint32 * d1, uint32 * d2, uint32 * d3) {
  uint32 a, b, c, d;
  a=*d0;b=*d1;c=*d2;d=*d3;
  a+=b; d=ROL32(d^a,16);
  c+=d; b=ROL32(b^c,12);
  a+=b; d=ROL32(d^a,8);
  c+=d; b=ROL32(b^c,7);
  *d0=a;*d1=b;*d2=c;*d3=d;
}
*/
#define round4_chacha_body(A,B,C,D) \
	paddd	A, B  ; \
	pxor	D, A  ; \
	ROL32(	D, 16); \
	paddd	C, D  ; \
	pxor	B, C  ; \
	ROL32(	B, 12); \
	paddd	A, B  ; \
	pxor	D, A  ; \
	ROL32(	D, 8) ; \
	paddd	C, D  ; \
	pxor	B, C  ; \
	ROL32(	B, 7)

/*
static void round16_chacha(uint32 * d) {
  round4_chacha(d+0, d+4, d+8, d+12);
  round4_chacha(d+1, d+5, d+9, d+13);
  round4_chacha(d+2, d+6, d+10, d+14);
  round4_chacha(d+3, d+7, d+11, d+15);
  round4_chacha(d+0, d+5, d+10, d+15);
  round4_chacha(d+1, d+6, d+11, d+12);
  round4_chacha(d+2, d+7, d+8, d+13);
  round4_chacha(d+3, d+4, d+9, d+14);
}
*/
#define round16_chacha_body \
	round4_chacha_body(xmm0,xmm1,xmm2,xmm3)      ; \
	BRL128(xmm1,12)      ; \
	BRL128(xmm2,8)       ; \
	BRL128(xmm3,4)       ; \
	round4_chacha_body(xmm0,xmm1,xmm2,xmm3)      ; \
	BRL128(xmm1,4)       ; \
	BRL128(xmm2,8)       ; \
	BRL128(xmm3,12)


#ifndef INLINE_ROUND16
#ifdef ALIGN_ROUND16
.balign 16
#endif
round16_chacha:
	round16_chacha_body
	ret
#define round16_chacha_call() call round16_chacha
#else
#define round16_chacha_call() round16_chacha_body
#endif


/*
extern void chacha20_step(uint32 * d) {
  uint32 tmp[16];
  uint32 i;
  for(i=0;i<16;i++) tmp[i]=d[i];
  for(i=0;i<10;i++) round16_chacha(tmp);
  for(i=0;i<16;i++) d[i]+=tmp[i];
}
*/
chacha20_step:
	mov	eax, [esp+4]
	movdqu	xmm0, [eax+0]
	movdqu	xmm1, [eax+16]
	movdqu	xmm2, [eax+32]
	movdqu	xmm3, [eax+48]
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	round16_chacha_call()
	movdqu	xmm4, [eax+0]
	movdqu	xmm5, [eax+16]
	movdqu	xmm6, [eax+32]
	movdqu	xmm7, [eax+48]
	paddd xmm0, xmm4
	paddd xmm1, xmm5
	paddd xmm2, xmm6
	paddd xmm3, xmm7
	movdqu	[eax+0], xmm0
	movdqu	[eax+16], xmm1
	movdqu	[eax+32], xmm2
	movdqu	[eax+48], xmm3
	ret



#define DATA(i) dword ptr [ecx+(i*4)]
#define DATADQ(i) [ecx+(i*4)]

salsa20_genrandom512:
	push	ebx
	mov	ecx,[esp+8]
	mov	edx,[esp+12]
	mov	ebx,[esp+16]
	mov	eax, [edx+0]
	mov	DATA(0), 0x61707865
	mov	DATA(1), 0x3320646E
	mov	DATA(2), 0x79622D32
	mov	DATA(3), 0x6B206574
	mov	DATA(13), eax
	mov	eax, [edx+4]
	mov	DATA(10), eax
	mov	eax, [edx+8]
	mov	DATA(7), eax
	mov	eax, [edx+12]
	mov	DATA(4), eax
	mov	eax, [ebx+0]
	mov	DATA(14), eax
	mov	eax, [ebx+4]
	mov	DATA(11), eax
	mov	ebx, [esp+20]
	mov	eax, [ebx+0]
	mov	DATA(8), eax
	mov	eax, [ebx+4]
	mov	DATA(5), eax
	mov	eax, [edx+16]
	mov	DATA(15), eax
	mov	eax, [edx+20]
	mov	DATA(12), eax
	mov	eax, [edx+24]
	mov	DATA(9), eax
	mov	eax, [edx+28]
	mov	DATA(6), eax
	pop	ebx
	push	ecx
	call	salsa20_step
	pop	ecx
	mov	eax, DATA(1)
	xchg	eax, DATA(5)
	xchg	eax, DATA(9)
	xchg	eax, DATA(13)
	mov	DATA(1), eax
	mov	eax, DATA(2)
	xchg	eax, DATA(10)
	mov	DATA(2), eax
	mov	eax, DATA(6)
	xchg	eax, DATA(14)
	mov	DATA(6), eax
	mov	eax, DATA(3)
	xchg	eax, DATA(15)
	xchg	eax, DATA(11)
	xchg	eax, DATA(7)
	mov	DATA(3), eax
	ret

chacha20_genrandom512:
	push	ebx
	mov	ecx,[esp+8]
	mov	edx,[esp+12]
	mov	ebx,[esp+20]
	mov	DATA(0), 0x61707865
	mov	DATA(1), 0x3320646E
	mov	DATA(2), 0x79622D32
	mov	DATA(3), 0x6B206574
	movdqu	xmm1, [edx+0]
	movdqu	DATADQ(4), xmm1
	movdqu	xmm2, [edx+16]
	movdqu	DATADQ(8), xmm2
	mov	eax, [ebx+0]
	mov	DATA(12), eax
	mov	eax, [ebx+4]
	mov	DATA(13), eax
	mov	ebx, [esp+16]
	mov	eax, [ebx+0]
	mov	DATA(14), eax
	mov	eax, [ebx+4]
	mov	DATA(15), eax
	pop	ebx
	jmp	chacha20_step

