/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * config.h - config file reader
 * Copyright (c) 2020-2021 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */
#ifndef SSLPROXY_CONFIG_H
#define SSLPROXY_CONFIG_H

#include <sys/types.h>
#include <fcl/types.h>

typedef struct {
  char const * base_dir; /* chdir to this dir made after the very start of proxy, so all other paths may use ./ as base */
  struct {
    char const * ip;
    uint16       port;
    uchar        raw_https;      /* accept raw https connections, not https-proxy */
  } listen;
  uint16       num_processes;
  struct {
    uint         keepalive;      /* http keep-alive timeout */
    uint         idle;           /* connection in data-transfer state but not sending anything */
  } connection_timeout;
  struct {
    uint16       minver;
    char const * cacert;
    char const * cakey;
    char const * proxykey;
    uchar        wildcard_certs;
    uchar        no_SAN;
  } local_ssl;
  struct {
    uint16       minver;
    char const * calist;
  } inet_ssl;
  struct {
    uint32       host_dns;
    uint16       port_dns;
    uint32       host_data;
    uint16       port_data;
  } rproxy;
  struct {
    char const * ssl_client_wrapper;
    char const * ssl_server_wrapper;
  } helpers;
} t_prefs;

extern t_prefs prefs;

int config_load(char const * fn);
void config_dump(void);

extern int str_to_size_t(char const * st, size_t * res);

#endif


#define DASHBOARD "./log/dashboard"
#define LOG_BASE "./log"
#define MAX_NUM_PROCESSES 1000

/* parameters for browser-side SSL connection */
#define SSL_MINVER prefs.local_ssl.minver     // 0x0303
#define SSL_CACERT prefs.local_ssl.cacert     // "./pem/cacert.pem"
#define SSL_CAKEY prefs.local_ssl.cakey       // "./pem/cakey.pem"
#define SSL_PROXYKEY prefs.local_ssl.proxykey // "./pem/proxykey.pem"

/* parameters for remote-side SSL connections */
#define SSL_R_MINVER prefs.inet_ssl.minver    // 0x0301
#define SSL_R_CALIST prefs.inet_ssl.calist    // "/etc/ssl/certs/ca-certificates.crt"

#ifndef _HTTP_METHODS_
#define _HTTP_METHODS_
#define M_GET 0
#define M_HEAD 1
#define M_POST 2
#define M_CONNECT 3
#define M_OPTIONS 4 /* it is nearly the same as HEAD but may be treated differently by scripts */
#endif

#ifndef _HTTP_STATES_
#define _HTTP_STATES_
#define state_closed 1
#define state_connected 2
#define state_https_pending2 4
#define state_request_pending 5
#define state_content_pending 6
#define state_process_request 7
#endif



extern char const * method_get_name(char m);

