/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * handle/cache.h - cache handler
 * Copyright (c) 2018-2021 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

typedef struct {
  unsigned int status;
  char const * ctype;
  char const * redir;
  int allow_xdomain;
  int offline;
  t_dynbuf data;
} t_cached_data;

extern void free_cached_data(t_cached_data * cd);

/* it eats *src in any case
   returns NULL on malformed *src
  */
extern t_cached_data * parse_cached_data(t_dynbuf * src, char const * host, char const * url, int cache);

/* store data by hash, return filename in hashp;
   filename is {HASH}.{IDX} or {HASH}.{IDX}.gz
   {HASH} = 40 characters of A-Za-z0-9_- (base64 of sha256[0:239])
   {IDX} = collision counter, 0 for first unique file with this {HASH}, 1 for second, etc, up to 65535
   .gz present if file was compressed
   */
extern int store_by_hash(int cache, t_dynbuf * content, char * hashp, int no_write);
