/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * handle/inc.http_sniff.c - http sniffer
 * Copyright (c) 2018-2020 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */


static void http_request_sniff(char https, char meth, char const * host, char const * url, t_http_headers const * hdrs, t_http_content const * data) {
  int fd, tmpr;
  unsigned int n;
  char const *name, *value;
  char tmpbuf[16384];
  unsigned long rbpos;
  
  /* TODO: eventually cleanup this code */
  fd = open(LOGFILE("sniff.log"), O_WRONLY|O_APPEND|O_CREAT, 0644);
  if(fd<0) return;
  flock(fd, LOCK_EX);
  
  dprintf(fd, "===================================================================\n");
  dprintf(fd, "[%s] %s %s://%s%s\n", get_timestr(), method_get_name(meth), https?"https":"http", host, url);
  for(n=0; n<hdrs->n; n++) {
    name = hdrs->h[n*2];
    value = hdrs->h[n*2+1];
    dprintf(fd, "%s: %s\n", name, value);
  }
  if(data && data->total) {
    dprintf(fd, "\n=== REQUEST BODY === %llu bytes ===\n", (unsigned long long)data->total);
    if(!data->use_fd) {
      write(fd, data->d, data->total);
    } else {
      if(lseek(data->fd, 0, SEEK_SET)<0) dprintf(fd, "***ERROR*** reading back request body from tmpfile failed, lseek() errno %d (%s)", errno ,strerror(errno));
      else for(rbpos=0; rbpos<data->total; ) {
        tmpr = read_safe(data->fd, tmpbuf, sizeof(tmpbuf));
        if(tmpr<0) { dprintf(fd,"***ERROR*** reading back request body from tmpfile failed, read() errno %d (%s), after %lu of %lu", errno, strerror(errno), (ulong)rbpos, (ulong)data->total); break; }
        if(tmpr==0) { dprintf(fd,"***ERROR*** reading back request body from tmpfile failed (EOF) after %lu of %lu", (ulong)rbpos, (ulong)data->total); break; }
        write(fd, tmpbuf, tmpr);
        rbpos += tmpr;
      }
    }
  }
  dprintf(fd, "\n===================================================================\n");
  close(fd);
}

static void http_proxied_request_sniff(t_dynbuf * req) {
  int fd;
  
  fd = open(LOGFILE("sniff.log"), O_WRONLY|O_APPEND|O_CREAT, 0644);
  if(fd<0) return;
  flock(fd, LOCK_EX);
  dprintf(fd, "=================================================================== PROXIED AS\n");
  write(fd, req->buf, req->len);
  dprintf(fd, "\n===================================================================\n");
  close(fd);
}

static void http_response_sniff(char https, char meth, char const * host, char const * url, char const * referer, char const * ctype, t_http_headers const * hdrs, t_dynbuf const * data) {
  static unsigned long count = 0;
  char fn[1000];

  int fd, tmpr;
  unsigned int n;
  char const *name, *value;
  char tmpbuf[16384];
  unsigned long rbpos;
  
  /* TODO: eventually cleanup this code */
  snprintf(fn, sizeof(fn), "tmp/%lu.%lu.%lu.bin", (unsigned long)time(NULL), (unsigned long)getpid(), count++);
  fd = open(fn, O_WRONLY|O_EXCL|O_CREAT, 0644);
  if(fd<0) return;
  
  dprintf(fd, "===================================================================\n");
  dprintf(fd, "[%s] %s %s://%s%s CTYPE=%s\n", get_timestr(), method_get_name(meth), https?"https":"http", host, url, ctype);
  dprintf(fd, "REF = %s\n", referer);
  for(n=0; n<hdrs->n; n++) {
    name = hdrs->h[n*2];
    value = hdrs->h[n*2+1];
    dprintf(fd, "%s: %s\n", name, value);
  }
  if(data && data->len) {
    dprintf(fd, "\n=== BODY === %llu bytes ===\n", (unsigned long long)data->len);
    write(fd, data->buf, data->len);
  }
  dprintf(fd, "\n===================================================================\n");
  close(fd);

}
