/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * handle/inc.log.c - proxy log writer
 * Copyright (c) 2018,2020 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */


#define QUERY_LOG 1
#define DATA_LOG 2
#define CLEANDATA_LOG 6

static void proxy_log(char const * summary,  char const * host, char meth, char const * url, char const * referer, int mode) {
  t_dynbuf b;
  dynbuf_init(&b, 128, 128, 65536);
  dynbuf_printstr(&b, summary);
  dynbuf_printstr(&b, " \"");
  dynbuf_printstr_cesc(&b, host);
  dynbuf_printstr(&b, "\" \"");
  dynbuf_printstr_cesc(&b, method_get_name(meth));
  dynbuf_printstr(&b, " ");
  dynbuf_printstr_cesc(&b, url);
  dynbuf_printstr(&b, " ");
  dynbuf_printstr_cesc(&b, referer);
  dynbuf_printstr(&b, "\"");
  dynbuf_nullterm(&b);
  if(mode&1) file_log_string(LOGFILE("query.log"), (char*)b.buf);
  if(mode&2) file_log_string(LOGFILE("data.log"), (char*)b.buf);
  if(mode&4) file_log_string(LOGFILE("cleandata.log"), (char*)b.buf);
  dynbuf_clear(&b);
}

static void cache_logf(char const * fmt, ...) {
  va_list arg;
  va_start(arg, fmt);
  file_log_vprintf(LOGFILE("cache.log"), fmt, arg);
  va_end(arg);
}

extern void error_logf(char const * fmt, ...) {
  va_list arg;
  va_start(arg, fmt);
  file_log_vprintf(LOGFILE("error.log"), fmt, arg);
  va_end(arg);
}

static void net_error_logf(char const * fmt, ...) {
  va_list arg;
  va_start(arg, fmt);
  file_log_vprintf(LOGFILE("net_error.log"), fmt, arg);
  va_end(arg);
}
