/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * handle/inc.proxy.c - main function that handles http request
 * Copyright (c) 2017-2022 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#define PROXYCTL "proxyctl"

static t_http_request $;

static void decode_chunked(unsigned char * buf, size_t * sz);
static void proxyctl_main(char meth, char const * url, char const * ref, t_http_content const * postdata);
#include "inc.proxyctl.c"

static int findsig(char const * st, size_t len, char const * sig) {
  size_t slen, pos;
  slen = strlen(sig);
  for(pos=0; pos+slen<=len; pos++) if(!memcmp(st+pos,sig,slen)) return 1;
  return 0;
}

static void proxy_main(void) {
  static char const *$host, *$cookie, *$referer, *$url, *$newurl, *$msg;
  static char $meth;
  static char $https, $is_raw, $is_sniff;
  static int $use_rproxy;
  static t_http_content const *$data;
  static t_http_headers $request_headers;
  static t_http_headers $response_headers;
  static unsigned int $status;
  char sstr[50];
  unsigned long long clen = 0;
  char const * replaced_ua;


  unsigned int n, m;
  int $r, is_damaged = 0;
  
  $meth = in_METH;
  if($meth!=M_GET && $meth!=M_POST && $meth!=M_HEAD && $meth!=M_OPTIONS) http400("bad method!");
  if(!($host=HDR("Host"))) http400("missing HTTP Host header!");
  $cookie = HDR("Cookie");
  $referer = HDR("Referer");
  $https = in_HTTPS;
  $url = in_RAW_URL; /* TODO: was $SERVER['REQUEST_URI'] - is it the same? */
  if($meth!=M_POST) $data=NULL; else $data=in_HC;

  if(!strcmp($host,PROXYCTL)) {
    if(!$https) {
      print_basic_headers(STATUS_301);
      printf("Content-Length: 0\r\nLocation: https://"PROXYCTL"/\r\n\r\n");
      exitreq();
    }
    proxyctl_main($meth,$url,$referer,$data);
    return;
  }
#if 0
//if($meth==M_POST) { header('Content-Type: text/plain'); var_dump($_SERVER['HTTP_CONTENT_TYPE']); var_dump($_POST); var_dump($_FILES); var_dump($HTTP_RAW_POST_DATA); var_dump(file_get_contents('php://input')); exit; }
function dump_all() {
  global $method,$host,$cookie,$referer,$https,$url,$data;
  header('Content-Type: text/plain');
  echo "METHOD\n";
  var_dump(method_get_name($meth));
  echo "HOST\n";
  var_dump($host);
  echo "COOKIE\n";
  var_dump($cookie);
  echo "REFERER\n";
  var_dump($referer);
  echo "HTTPS\n";
  var_dump($https);
  echo "URL\n";
  var_dump($url);
  echo "DATA\n";
  var_dump($data);
  echo "DONE\n";
  exit;
}
  if($host==='proxy.m1089.net') {
    var_dump(get_http_headers());
    dump_all();
  }
#endif


  $is_raw = 0;

  if($is_sniff = is_sniff($host, $url)) {
    http_request_sniff($https, $meth, $host, $url, in_HH, $data);
  }

  if($newurl = do_pre_redirect_url($meth, $host, $url)) {
    proxy_log(tmp_sprintf("REDIR-%s",$newurl), $host, $meth, $url, $referer, QUERY_LOG|DATA_LOG);
    print_basic_headers(STATUS_307);
    printf("Content-Length: 0\r\nLocation: %s\r\n\r\n",$newurl);
    exitreq();
  }

  int blockr;
  if((blockr=is_blocked($host, $url))) {
    $msg=tmp_sprintf("bad %s blocked: %s%s", rules_match_name(blockr), $host, (blockr>=4)?$url:"");
    proxy_log("BLOCKED", $host, $meth, $url, $referer, QUERY_LOG|DATA_LOG);
//    print_basic_headers(STATUS_404);
    print_basic_headers(STATUS_500);
    printf("Content-Length: %lu\r\nContent-Type: text/plain\r\n\r\n%s", strlen($msg), $msg);
    exitreq();
  }

  if($newurl = do_redirect_url($meth, $host, $url)) {
    proxy_log(tmp_sprintf("REDIR-%s",$newurl), $host, $meth, $url, $referer, QUERY_LOG|DATA_LOG);
    print_basic_headers(STATUS_307);
    printf("Content-Length: 0\r\nLocation: %s\r\n\r\n",$newurl);
    exitreq();
  }
  do_rewrite_url(&$meth, &$host, &$url);
  if($meth!=M_POST) $data=NULL; // re-check after possible rewrite
  if($meth!=M_POST && $meth!=M_OPTIONS) {
    t_cached_data * cd;
    if(cd = get_cached_data($host, $url)) {
      snprintf(sstr,sizeof(sstr),"%s-%u-%lu",(cd->offline?"OFFLINE":"CACHED"),cd->status,(ulong)cd->data.len);
      proxy_log(sstr, $host, $meth, $url, $referer, QUERY_LOG|DATA_LOG);
      switch(cd->status) {
      default:  print_basic_headers(STATUS_200); break;
      case 404: print_basic_headers(STATUS_404); break;
      case 301: print_basic_headers(STATUS_301); break;
      case 302: print_basic_headers(STATUS_302); break;
      case 307: print_basic_headers(STATUS_307); break;
      }
      if(cd->redir) printf("Content-Length: 0\r\nLocation: %s\r\n",cd->redir);
      else {
        printf("Content-Type: %s\r\n",cd->ctype);
        printf("Content-Length: %lu\r\n",(ulong)cd->data.len);
        if(cd->allow_xdomain) printf("Access-Control-Allow-Origin: *\r\n");
        printf("\r\n");
        outbytes(cd->data.buf, cd->data.len);
      }
      free_cached_data(cd);
      return;
    }
  }
  $is_raw = is_raw($host, $url);
  $use_rproxy = is_rproxy($host, $url);

  proxy_log("NOTCACHED", $host, $meth, $url, $referer, QUERY_LOG);
  replaced_ua = replace_user_agent($host, $url);
  
  for(n=m=0; n<in_HH->n; n++) {
    char const *name, *value;
    name = in_HH->h[n*2];
    value = in_HH->h[n*2+1];
    if(!strcmp(name,"Referer") && is_strip_referer($host, $url, value)) continue;
    if(!strcmp(name,"Cookie") && block_cookie($host, $url)) continue;
    if(!strcmp(name,"User-Agent") && replaced_ua) continue;
    $request_headers.h[m*2] = (char*)name;
    $request_headers.h[m*2+1] = (char*)value;
    m++;
  }
  if(replaced_ua && m<MAX_HEADERS) {
    $request_headers.h[m*2] = "User-Agent";
    $request_headers.h[m*2+1] = (char*)replaced_ua;/* this struct won't be free'd nor modified */
    m++;
  }
  $request_headers.n = m;

  $r = http_request(&$, $https, $meth, $host, $url, &$request_headers, $data, $is_raw, $use_rproxy);
  if($is_sniff) {
    http_proxied_request_sniff(&$.req);
  }
  if($.R == err_net) {
    net_error_logf("error '%s' for request '%s %s %s'", $.errmsg, method_get_name($meth), $host, $url);
    proxy_log("ERROR", $host, $meth, $url, $referer, CLEANDATA_LOG);
    http502c("GOT A ERROR WHILE DOING PROXY REQUEST", $.errmsg, NULL, NULL, $.certs);
  }
  if($.R == err_bad) {
    proxy_log("BADREPLY", $host, $meth, $url, $referer, CLEANDATA_LOG);
    http502("GOT BAD REPLY FROM SERVER WHILE DOING PROXY REQUEST", $.errmsg, &$.req, &$.resp);
  }
  if($.R == err_badst) {
    proxy_log(tmp_sprintf("BADSTATUS-%u",$.http), $host, $meth, $url, $referer, CLEANDATA_LOG);
    http502("GOT BAD STATUS ($status) FROM SERVER WHILE DOING PROXY REQUEST", $.errmsg, &$.req, &$.resp);
  }
  if($.R!=status_ok && $.R!=status_raw) {
    proxy_log("UNKNOWN", $host, $meth, $url, $referer, CLEANDATA_LOG);
    http502("GOT UNKNOWN FROM SERVER WHILE DOING PROXY REQUEST", $.errmsg, &$.req, &$.resp);
  }
  $status=$.http;

  char $is_gzip, $is_chunked, $is_acao;
  char const *$ctype, *$redir, *hname, *hvalue;
  
  $is_gzip = 0;
  $is_chunked = 0;
  $is_acao = 0;
  $ctype = "text/html";
  $redir = "";
  for(n=m=0; n<$.hdrs.n; n++) {
    hname = $.hdrs.h[n*2];
    hvalue = $.hdrs.h[n*2+1];
//  fprintf(stderr,"%s: %s\n",hname,hvalue);
    if(!strcmp(hname,"Content-Type")) {
      will_free((char*)($ctype=xstrdup(hvalue)));
      strtolower((char*)$ctype);
      dashboard_set_ctype($ctype);
      continue;
    }
    if(!strcmp(hname,"Access-Control-Allow-Origin") && !strcmp(hvalue,"*")) $is_acao=1;
    if(!strcmp(hname,"Content-Length")) {
      clen = strtoull(hvalue,NULL,10);
      dashboard_set_clen(clen);
      continue;
    }
    if(block_cookie($host, $url) && !strcmp(hname,"Set-Cookie")) continue;
    if(!strcmp(hname,"Location")) will_free((char*)($redir=xstrdup(hvalue)));
    if(!strcmp(hname,"Content-Encoding")) {
      if(!strcasecmp(hvalue,"gzip")) $is_gzip=1;
      else if(!strcasecmp(hvalue,"none")) $is_gzip=0;
      else {
        proxy_log("BADCOMPR", $host, $meth, $url, $referer, CLEANDATA_LOG);
        http502("BAD COMPRESSION MODE FROM SERVER", hvalue, &$.req, &$.resp);
      }
      continue;
    }
    if(!strcmp(hname,"Transfer-Encoding")) {
      if(!strcasecmp(hvalue,"chunked")) $is_chunked=1;
      else {
        proxy_log("BADCHUNK", $host, $meth, $url, $referer, CLEANDATA_LOG);
        http502("BAD TRANSFER-ENCODING FROM SERVER", hvalue, &$.req, &$.resp);
      }
      continue;
    }
    $response_headers.h[m*2] = (char*)hname;
    $response_headers.h[m*2+1] = (char*)hvalue;
    m++;
  }
  $response_headers.n = m;


  // $.hdrs            === $hdrs[]  is original headers
  // $response_headers === $hdrs2[] is filtered headers
  
  if($.R==status_raw) {
#ifdef TRY_TO_REENCODE_RAW /* TODO: $.content was t_dynbuf; now it is t_http_content and the code below should be revised before use */
  // TODO: allow gzip too
    if($is_gzip) http502("BAD REPLY (gzipped reply was not allowed)", NULL, NULL, NULL);
//    if($is_chunked) http502("BAD REPLY (chunked reply was not allowed)", NULL, NULL, NULL);
    snprintf(sstr,sizeof(sstr),"GOTRAW-%u",$status);
    proxy_log(sstr, $host, $meth, $url, $referer, CLEANDATA_LOG);
    printf("%s\r\nConnection: close\r\n", $.statusline); conn_close=1;
    for(n=0; n<$.hdrs.n; n++) {
      hname = $.hdrs.h[n*2];
      hvalue = $.hdrs.h[n*2+1];
      if(!strcmp(hname,"Content-Encoding")) continue;
      if(!strcmp(hname,"Transfer-Encoding")) continue;
      if(!strcmp(hname,"Connection")) continue;
      if(block_cookie($host, $url) && !strcmp(hname,"Set-Cookie")) continue;
      printf("%s: %s\r\n", hname, hvalue);
      // content-type and content-length - using original values if any
    }
    printf("\r\n");
    {
      int sock, rrr;
      char sbuf[10240];
      sock = $.socket;
      if(!$is_chunked) {
        outbytes($.content.buf, $.content.len);
        while(1) {
          rrr = lst_conn_recv(sock, sbuf, sizeof(sbuf));
          if(rrr<0) break; /* TODO: proper error handling */
          if(rrr>0) outbytes(sbuf, rrr);
        }
      } else {
        char $wait, $state;
        size_t pp;
        unsigned long remsz;
        $wait = 0;
        $state = 0;
        while(1) {
          if($wait) {
            rrr = lst_conn_recv(sock, sbuf, sizeof(sbuf));
            if(rrr<0) { proxy_log("TRUNCCHUNK", $host, $meth, $url, $referer, CLEANDATA_LOG); break; }
            if(!rrr) continue;
            if(dynbuf_write(&$.content, sbuf, rrrr)<0) assert(!"dynbuf somehow overflowed"); // it shouldn't overflow due to code below
            $wait = 0;
          }
          if($state==0) {
            for(pp=0; pp+1<$.content.len; pp++) if($content.buf[pp]=='\r' && $content.buf[pp+1]=='\n') goto found1;
            if(pp>8) { proxy_log("WRONGCHUNK", $host, $meth, $url, $referer, CLEANDATA_LOG); break; }
            $wait=1; continue;
            found1:
            $.content.buf[pp] = 0;
            if(str_hex_to_ulong((char*)$.content.buf, &remsz)<0) { proxy_log("WRONGCHUNK", $host, $meth, $url, $referer, CLEANDATA_LOG); break; }
            dynbuf_drain(&$.content, pp+2);
            $state = 1;
            if(!remsz) break; /* ok, done */
          }
          if($state==1) {
            if(remsz) {
              pp = $.content.len;
              if(pp>remsz) pp=remsz;
              outbytes($.content.buf, pp);
              dynbuf_drain(&$.content, pp);
              remsz -= pp;
            }
            if(!remsz) {
              if($.content.len<2) { $wait=1; continue; }
              if($.content.buf[0]!='\r' || $.content.buf[1]!='\n') { proxy_log("WRONGCHUNK", $host, $meth, $url, $referer, CLEANDATA_LOG); break; }
              dynbuf_drain(&$.content, 2);
              $state = 0;
            }
          }
        }
      }
      close(sock);
    }
#else
    /* just send everything to client */
    snprintf(sstr,sizeof(sstr),"GOTRAW-%u",$status);
    proxy_log(sstr, $host, $meth, $url, $referer, CLEANDATA_LOG);
    printf("%s\r\nConnection: close\r\n", $.statusline); conn_close=1;
    for(n=0; n<$.hdrs.n; n++) {
      hname = $.hdrs.h[n*2];
      hvalue = $.hdrs.h[n*2+1];
      if(!strcmp(hname,"Connection")) continue;
      if(block_cookie($host, $url) && !strcmp(hname,"Set-Cookie")) continue;
      printf("%s: %s\r\n", hname, hvalue);
    }
    printf("\r\n");
    {
      ulong dlen_total;
      int sock, rrr;
      char sbuf[10240];
      sock = $.socket;
assert($.content.d && !$.content.use_fd);
      outbytes($.content.d, $.content.cur);
      dlen_total = $.content.cur;
      while(1) {
        dashboard_set_dlen(dlen_total);
        rrr = lst_conn_recv(sock, sbuf, sizeof(sbuf));
        if(rrr<0) break; /* TODO: proper error handling */
        if(rrr>0) { outbytes(sbuf, rrr); dlen_total+=rrr;}
        if(!$is_chunked && dlen_total>=$.clen) break;
      }
      close(sock);
    }
#endif
    return;
  }

t_dynbuf rcontent;
assert($.content.d && !$.content.use_fd);
dynbuf_init_data(&rcontent, $.content.d, $.content.cur);

  if(clen && rcontent.len!=clen) is_damaged = 1;
  

  if($is_gzip) {
    t_dynbuf decomp;
    int z_err, gr;
    dynbuf_init(&decomp, rcontent.len, 8192, 10000000); /* TODO: overflow of these 10M will be silent DMG- becasue there is no place to store error message */
    if((gr=gzip_decompress(&decomp, &rcontent, &z_err))<0) {
      /* TODO: proper error handling */
      dynbuf_printf(&decomp, "!!! decompression error %d %d %lu %lu!!!",gr,z_err,(unsigned long)rcontent.len,(unsigned long)$.content.cur);
      is_damaged = 1;
    }
    dynbuf_clear(&rcontent);
    rcontent = decomp;
  }

  if(rcontent.len<2000 && findsig((char*)rcontent.buf, rcontent.len, "mobilebanner.ru")) is_damaged = 1;

  if($is_sniff) {
    http_response_sniff($https, $meth, $host, $url, $referer, $ctype, &$.hdrs, &rcontent);
  }
  if($status==200 && $meth==M_GET) got_data($host, $url, $ctype, &rcontent, $is_acao, is_damaged); //else file_put_contents('./tmp/'.time().'.'.mt_rand(0,99), "method_get_name($meth) $url\n$ctype\n{---}\n$data\n{---}\n$contents");
  proxy_log(tmp_sprintf("%s-%u-%lu-%s",is_damaged?"DMG":"GOT",$status,(unsigned long)rcontent.len,$redir), $host, $meth, $url, $referer, CLEANDATA_LOG);
  do_postproc($meth, $host, $url, &$ctype, &rcontent);

  printf("%s\r\n", $.statusline);
  for(n=0; n<$response_headers.n; n++) printf("%s: %s\r\n", $response_headers.h[n*2], $response_headers.h[n*2+1]);
  printf("Content-Type: %s\r\n", $ctype);
  printf("Content-Length: %lu\r\n", (unsigned long)rcontent.len);
  printf("\r\n");
  outbytes(rcontent.buf, rcontent.len);
  dynbuf_clear(&rcontent);
}





// Credits to @flowfree (http://stackoverflow.com/users/1396314/flowfree) for this function.
// http://stackoverflow.com/questions/10793017/how-to-easily-decode-http-chunked-encoded-string-when-making-raw-http-request
static void decode_chunked(unsigned char * buf, size_t * sz) {
  /* translated from php; has no error checking at all; anyway this function should be removed and replaced with proper decoding such as in status_raw branch */
  unsigned char * dest;
  unsigned char * b2;
  unsigned long tmpsz;
  char c;
  size_t avail, wrote, pos;
  avail = *sz;
  wrote = 0;
  dest = buf;
  while(avail) {
    while(avail && (*buf==' ' || *buf=='\r' || *buf=='\n')) { buf++; avail--; }
    for(pos=0; pos+2<=avail; pos++) if(buf[pos]=='\r' && buf[pos+1]=='\n') break;
    if(pos+2>avail) break;
    buf[pos]=0;
    if(str_hex_to_ulong((char*)buf, &tmpsz)<0) break;
    buf += pos+2; avail -= pos+2;
    if(tmpsz>avail) { bcopy(buf, dest, avail); *sz=wrote+avail; return; }
    bcopy(buf, dest, tmpsz);
    dest += tmpsz;
    wrote += tmpsz;
    buf += tmpsz;
    avail -= tmpsz;
  }
  *sz = wrote;
  return;
}


