/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * handle/inc.reply.c - standard http replies templates
 * Copyright (c) 2018, 2022 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */


static void http400(char const * msg) { /* "Bad Request" */
  print_basic_headers(STATUS_400);
  printf("Content-Length: %u\r\nContent-Type: text/plain\r\n\r\n%s", strlen(msg), msg);
  exitreq();
}

static void http500(char const * msg) { /* "Internal Server Error" */
  print_basic_headers(STATUS_500);
  printf("Content-Length: %u\r\nContent-Type: text/plain\r\n\r\n%s", strlen(msg), msg);
  exitreq();
}

#define http502(msg,e,r,rr) http502c(msg,e,r,rr,NULL)
static void http502c(char const * msg, char const * errmsg, t_dynbuf * req, t_dynbuf * resp, char const * certs) {
  t_dynbuf t;
  size_t j;
  dynbuf_init(&t, 1024, 1024, 32767);
  dynbuf_printstr(&t, msg);
  if(errmsg) {
    dynbuf_printstr(&t, "<br><br>");
    dynbuf_printstr_hsc(&t, errmsg);
  }
  if(req) {
    dynbuf_printstr(&t, "<hr>Request:<textarea>");
    dynbuf_printbytes_hsc(&t, req->buf, req->len);
    dynbuf_printstr(&t, "</textarea>");
  }
  if(resp) {
    dynbuf_printstr(&t, "<hr>Reply:<textarea>");
    dynbuf_printbytes_hsc(&t, resp->buf, resp->len);
    dynbuf_printstr(&t, "</textarea>");
  }
  if(certs) {
    dynbuf_printstr(&t, "<hr>Certs (<a href=\"about:certificate?");
    /* TODO: proper parser */
    while(1) {
      if(*certs<'0' || *certs>'9') break;
      if(certs[1]!='=') break;
      certs += 2;
      for(j=0; certs[j] && certs[j]!='\n'; j++);
      if(certs[j]!='\n') break;
      dynbuf_printstr(&t, "&amp;cert=");
      dynbuf_printbytes_uesc(&t, certs, j);
      certs += j+1;
    }
    dynbuf_printstr(&t, "\">view in firefox</a>)");
  }
  print_basic_headers(STATUS_502);
  printf("Content-Length: %u\r\nContent-Type: text/html\r\n\r\n", (unsigned)t.len);
  outbytes(t.buf, t.len);
  dynbuf_clear(&t);
  exitreq();
}

