/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * http_packet.h - http protocol handler
 * Copyright (c) 2018-2020 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#ifndef _HTTP_METHODS_
#define _HTTP_METHODS_
#define M_GET 0
#define M_HEAD 1
#define M_POST 2
#define M_CONNECT 3
#define M_OPTIONS 4 /* it is nearly the same as HEAD but may be treated differently by scripts */
#endif

#ifndef _HTTP_STATES_
#define _HTTP_STATES_
#define state_closed 1
#define state_connected 2
#define state_https_pending2 4
#define state_request_pending 5
#define state_content_pending 6
#define state_process_request 7
#endif

#define MAX_HEADERS      255
#define MAX_HEADER_NAME  64
#define MAX_HEADER_VALUE (8192*8) /* TODO: go back 8192 after cleaning up wrong cookies from archive.org */

typedef struct {
  unsigned int n;
  char * h[MAX_HEADERS*2];
} t_http_headers;

extern int hh_add(t_http_headers * hh, char const * text);
extern int hh_parse(char const * text, char * * r_name, char * * r_value);
extern void hh_clear(t_http_headers * hh);
extern char const * hh_get(t_http_headers const * hh, char const * name);

#define INIT_MEM_CONTENT 32768 /* must fit ulong and size_t */
#define MAX_MEM_CONTENT 10485760 /* must fit ulong and size_t; this is 10M currently to prevent using files for responses due to incompleteness */
#define MAX_FILE_CONTENT 2000000000 /* must fit ulong and off_t */
typedef struct {
  unsigned char * d;
  unsigned long alloc, cur, total;
  int use_fd, fd;
  /* total=0 means dynamic size starting from INIT_MEM_CONTENT and limited to MAX_FILE_CONTENT
     total>0 means fixed limit */
} t_http_content;

extern int hc_init(t_http_content * hc, unsigned long max_size);
extern void hc_clear(t_http_content * hc);
extern int hc_add(t_http_content * hc, unsigned char const * buf, size_t len);


typedef struct {
  int method;
  char v1, v2;
  char  mstr[16]; /* for bad-method debug message only! */
  char  raw_url[4096];
  t_http_headers hh;
  unsigned long clen;
  t_http_content hc;
} t_http_packet;

extern int hp_parse_firstline(t_http_packet * hp, char * fl);
