/*
 * This file is part of fproxy - multipurpose http(s) proxy server
 * nslookup.c - dns resolver
 * Copyright (c) 2018-2019 firk (firk@cantconnect.ru)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The origin of this software must not be misrepresented; you must
 *    not claim that you wrote the original software.
 * 4. Altered versions in any form must be plainly marked as such, and
 *    must not be misinterpreted as being the original software.
 *
 * This software is provided by the author and contributors `as is'
 * without any express or implied warranty.
 */

#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <libsimpletcp/libsimpletcp.h>
#include "helpers/remote.h"
#include "util.h"
#include "nslookup.h"

static uint32 dns_proxy_addr = 0;
static uint16 dns_proxy_port = 0;
static int dns_proxy_fd = -1;

extern void nslookup_set_proxy(uint32 addr, uint16 port) {
  dns_proxy_addr = addr;
  dns_proxy_port = port;
  if(dns_proxy_fd>=0) { lst_conn_close(dns_proxy_fd,0); dns_proxy_fd = -1; }
}

/* TODO: way to specifiy non-system-wide DNS server */

/* return: -1=error; 0=ok, addr stored, port MAY BE stored but may be left unchanged */
extern int nslookup(char const * hostname, uint32 * addr, uint16 * port) {
  struct hostent * h;
  char * portstr;
  char * hostname2;
  int r;
  uint32 r32;
  
  r = lst_parse_addrstr(hostname, addr, port);
  if(r==1 || r==3) return 0; /* addr or addr:port parsed ok */
  hostname2 = xstrdup(hostname);
  if(portstr=strchr(hostname2,':')) {
    r = lst_parse_addrstr(portstr, addr, port);
    if(r!=2) { free(hostname2); return -1; /* bad port */ }
    *portstr = 0;
  }
  if(strlen(hostname2)>255) { free(hostname2); return -1; }
  if(!dns_proxy_port) {
    h = gethostbyname(hostname2);
    free(hostname2);
    if(!h || h->h_addrtype!=AF_INET || !h->h_addr_list || !h->h_addr_list[0]) return -1;
    *addr = ntohl(*(uint32*)(h->h_addr_list[0]));
  } else {
    r32 = lst_remote_nslookup(&dns_proxy_fd, dns_proxy_addr, dns_proxy_port, hostname2);
    if(r32==(uint32)-1) return -1;
    *addr = r32;
  }
  return 0;
}

/*
<?php
$dns_server = '195.34.32.116'; //'192.168.1.1';
//$dns_server='10.77.48.115'; // megafon
function nslookup($host) {
//return $host;
  global $dns_server;
  $res = `/usr/bin/nslookup $host $dns_server 2>&1`;
  $res = explode("\n",trim($res));
  $rr=FALSE;
  foreach($res as $st) {
    $st = trim($st);
    if(substr($st,0,9)==='Address: ') $rr=trim(substr($st,9));
  }
  if($rr===FALSE) { echo 'nslookup('.htmlspecialchars($host).') failed'; exit; }

  return $rr;
}

*/
