/* (c) firk 2012-2016
   please respect the authorship
   feel free to use this code for anything */
/* 2016-11-27 incompatible change: addrs now stored in
   proper host byte order, not in reversed */
#ifndef INCLUDED_LIBSIMPLETCP_H
#define INCLUDED_LIBSIMPLETCP_H

#ifdef BUILD_LIBSIMPLETCP
#define EXPORTED_LIBSIMPLETCP __attribute__((visibility("default")))
#else
#define EXPORTED_LIBSIMPLETCP
#endif

#ifndef INCLUDED_FCL_TYPES_H
typedef unsigned char uint8, uchar;
typedef unsigned short uint16;
typedef unsigned int uint32;
#endif


EXPORTED_LIBSIMPLETCP extern int    lst_get_errcode(void);

#define LST_INADDR_ANY      0
#define LST_INADDR_NONE     0xFFFFFFFF
#define LST_INADDR_LOOPBACK 0x7F000001
EXPORTED_LIBSIMPLETCP extern uint32 lst_parse_ipstr(char const * ipstr);
EXPORTED_LIBSIMPLETCP extern int    lst_parse_addrstr(char const * st, uint32 * ip_r, uint16 * port_r);
EXPORTED_LIBSIMPLETCP extern uint32 lst_parse_domainstr(char const * ipstr);
EXPORTED_LIBSIMPLETCP extern char * lst_print_ip(char * dest, uint32 addr);
EXPORTED_LIBSIMPLETCP extern char * lst_print_ip_port(char * dest, uint32 addr, uint16 port);

#define LST_FLAG_NONBLOCK_BEFORE 1  /* open: nonblocking connect process and connection */
#define LST_FLAG_NONBLOCK_AFTER  2  /* open: nonblocking connection after it connected */
#define LST_FLAG_GRACEFUL        4  /* close: do shutdown before closesocket */
#define LST_FLAG_IGNORE_ERRORS   8  /* close: don't change errorcode even if errors occured during socket close */
#define LST_FLAG_REUSE_ADDR      16 /* listen: actiate SO_REUSEADDR */

EXPORTED_LIBSIMPLETCP extern int    lst_init(void);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_open(uint32 addr, uint16 port, uint32 flags);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_open_from(uint32 b_addr, uint16 b_port, uint32 addr, uint16 port, uint32 flags);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_listen(uint32 addr, uint16 port, uint16 backlog, uint32 flags);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_accept(int lsock, uint32 * addr, uint16 * port, uint32 flags);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_open_unix(char const * path, uint32 flags);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_listen_unix(char const * path, uint32 backlog, uint32 flags);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_accept_unix(int lsock, uint32 flags);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_set_nonblock(int sock);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_set_reuseaddr(int sock);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_set_keepalive(int sock);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_set_tcp_nodelay(int sock);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_is_readable(int sock);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_is_writable(int sock);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_wait_readable(int sock, unsigned int ms);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_wait_writable(int sock, unsigned int ms);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_wait_connection(int sock, unsigned int ms);
EXPORTED_LIBSIMPLETCP extern void   lst_conn_close(int sock, uint32 flags);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_send(int sock, void const * buf, int bufsize);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_send_all(int sock, void const * buf, int bufsize);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_recv(int sock, void * buf, int bufsize);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_recv_all(int sock, void * buf, int bufsize);
#ifndef HAVE_NOT_SIZE_T
EXPORTED_LIBSIMPLETCP extern int    lst_conn_s_send(int sock, void const * buf, size_t bufsize, size_t * rsize);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_s_send_all(int sock, void const * buf, size_t bufsize);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_s_recv(int sock, void * buf, size_t bufsize, size_t * rsize);
EXPORTED_LIBSIMPLETCP extern int    lst_conn_s_recv_all(int sock, void * buf, size_t bufsize, size_t * rsize);
#endif


#define LST_ENOERROR    0
#define LST_EWOULDBLOCK 1  /* operation will block but it is non-blocking socket */
#define LST_EINTR       2  /* opeation interrupted before done */
#define LST_ENOMEM      3  /* out of memory */
#define LST_EBADF       4  /* bad socket id */
#define LST_ECONNRESET  5  /* connection can't be established or already closed */
#define LST_EADDRINUSE  6  /* address already in use when bind */
#define LST_EFAULT      7  /* bad pointer */
#define LST_EINVAL      8  /* completely bad arguments (not supported options etc) */
#define LST_ELIMIT      9  /* max num of sockets reached */
#define LST_EUNKNOWN    10 /* unclassified failure */

#endif


